#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "rpu2image4xi.py">
#<abstract4xi abstract = "RPU 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "rpu2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'RPU2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4rpu',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4rpu',              # ACK 5.3.4 p90
            'copy2buffertoimage4rpu',              # ACK 5.3.4 p92
            'create2image4rpu',                    # ACK 5.31  p76            

            'get2imageinfo4rpu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4rpu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4rpu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4rpu',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4rpu',     # ACK modernized        
            'manoeuvre2copybuffertoimage4rpu',     # ACK modernized
            'manoeuvre2mapimage4rpu',              # ACK modernized                                           
            'manoeuvre2readimage4rpu',             # ACK modernized
            'manoeuvre2writeimage4rpu',            # ACK modernized            
            'map2image4rpu',                       # ACK 5.3.5 p94 
            
            'read2image4rpu',                      # ACK 5.3.3 p84  
            'retain2image4rpu',                    # ACK 5.4.1 p99
            'release2image4rpu',                   # ACK 5.4.1 p99

            'set2imagecallback4rpu',               # ACK modernized
            'set2image4rpu',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4rpu',                     # ACK modernized 
                                    
            'wait2image4rpu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4rpu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from rpu2buffer4xi import RPU2Buffer4Xi
from rpu2buffer4xi import create2buffer4rpu
from rpu2buffer4xi import create2subbuffer4rpu
from rpu2buffer4xi import get2bufferinfo4rpu
from rpu2buffer4xi import release2buffer4rpu
from rpu2buffer4xi import retain2buffer4rpu
from rpu2buffer4xi import set2buffer4rpu
from rpu2buffer4xi import trace2buffer4rpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from rpu2commandqueue4xi import RPU2CommandQueue4Xi
from rpu2commandqueue4xi import create2commandqueue4rpu 
from rpu2commandqueue4xi import flush2commandqueue4rpu
from rpu2commandqueue4xi import finish2commandqueue4rpu
from rpu2commandqueue4xi import get2commandqueueinfo4rpu
from rpu2commandqueue4xi import release2commandqueue4rpu
from rpu2commandqueue4xi import retain2commandqueue4rpu
from rpu2commandqueue4xi import set2commandqueue4rpu
from rpu2commandqueue4xi import trace2commandqueue4rpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from rpu2context4xi import RPU2Context4Xi 
from rpu2context4xi import create2context4rpu
from rpu2context4xi import get2context4rpu
from rpu2context4xi import release2context4rpu
from rpu2context4xi import retain2context4rpu
from rpu2context4xi import set2context4rpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from rpu2device4xi import RPU4Xi
from rpu2device4xi import init2rpu4xi 
from rpu2device4xi import set2rpu4xi
from rpu2device4xi import get2rpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from rpu2event4xi import RPU2Event4Xi
from rpu2event4xi import create2event4rpu
from rpu2event4xi import get2eventinfo4rpu
from rpu2event4xi import release2event4rpu
from rpu2event4xi import retain2event4rpu
from rpu2event4xi import set2eventcallback4rpu 
from rpu2event4xi import set2event4rpu
from rpu2event4xi import trace2event4rpu
from rpu2event4xi import wait2event4rpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global rpu2image4xi                                                     # 2D-IMAGES FOR RPU

global rpu2trace4xi                                                     # TRACE

global rpu2buffer4xi                                                    # BUFFER FOR RPU

global control2readimage4rpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH RPU  
global state2read4rpu                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4rpu                                                # MAX FOR STATE
global ack2readinfo4rpu                                                 # ACK FOR IMAGE INFO
global message2readget4rpu                                              # QUEUE GET
global message2readput4rpu                                              # QUEUE PUT
global message2readreceive4rpu                                          # PIPE RECEIVE
global message2readsend4rpu                                             # PIPE SEND

global control2writeimage4rpu                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH RPU 
global state2write4rpu                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4rpu                                               # MAX FOR STATE
global ack2writeinfo4rpu                                                # ACK FOR IMAGE INFO
global message2writeget4rpu                                             # QUEUE GET
global message2writeput4rpu                                             # QUEUE PUT
global message2writereceive4rpu                                         # PIPE RECEIVE
global message2writesend4rpu                                            # PIPE SEND

global control2copyimage4rpu                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH RPU 
global state2copyimage4rpu                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4rpu                                           # MAX FOR STATE
global ack2copyimageinfo4rpu                                            # ACK FOR IMAGE INFO
global message2copyimageget4rpu                                         # QUEUE GET
global message2copyimageput4rpu                                         # QUEUE PUT
global message2copyimagereceive4rpu                                     # PIPE RECEIVE
global message2copyimagesend4rpu                                        # PIPE SEND

global control2copyimagetobuffer4rpu                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH RPU 
global state2copyimagetobuffer4rpu                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4rpu                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4rpu                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4rpu                                 # QUEUE GET
global message2copyimagetobufferput4rpu                                 # QUEUE PUT
global message2copyimagetobufferreceive4rpu                             # PIPE RECEIVE
global message2copyimagetobuffersend4rpu                                # PIPE SEND

global control2copybuffertoimage4rpu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH RPU 
global state2copybuffertoimage4rpu                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4rpu                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4rpu                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4rpu                                 # QUEUE GET
global message2copybuffertoimageput4rpu                                 # QUEUE PUT
global message2copybuffertoimagereceive4rpu                             # PIPE RECEIVE
global message2copybuffertoimagesend4rpu                                # PIPE SEND                      

global control2mapimage4rpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH RPU  
global state2map4rpu                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4rpu                                                # MAX FOR STATE
global ack2mapinfo4rpu                                                 # ACK FOR IMAGE INFO
global message2mapget4rpu                                              # QUEUE GET
global message2mapput4rpu                                              # QUEUE PUT
global message2mapreceive4rpu                                          # PIPE RECEIVE
global message2mapsend4rpu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4rpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF RPU2IMAGE4XI5TEST - TRYING TO TRACE RPU: ', time4xi())      
#   print('=' *33)      
#   trace2rpu = None
#   try:
#       trace2rpu = trace2image4rpu() 
#       print('(Step 15B/25) OF RPU2IMAGE4XI5TEST - TRACE2RPU: ', trace2rpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4rpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4rpu', 'METHOD': 'trace2image4rpu'}
#   except:
#       print('(Step 15Z1/25) OF RPU2IMAGE4XI5TEST *** FAILED TO TRACE RPU: ', trace2rpu)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4rpu(task4trace2rpu=None):  

    '''
    creates callback to rpu2image4xi
    '''
    
    global rpu2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4rpu" 
    method2print = "trace2image4rpu"

    message2print = "{(Step 1A/33) started" + "; trace2image4rpu-started: " + str(trace2image4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4rpu at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2rpu-to-trace2image4rpu: " + str(task4trace2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    rpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2rpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY RPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2rpu = {}                                                                                            
            
        task4trace2rpu['CLASS']   = "trace2image4rpu"                                                               # RPUIZE ZPU TRACE!
        task4trace2rpu['METHOD']  = "trace2image4rpu" 
        task4trace2rpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4rpu-failed " + str(rpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2rpu-set-by-trace2image4rpu to: " + str(task4trace2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4rpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4rpu',
        #  'METHOD': 'trace2image4rpu'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY RPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2rpu-try-by-trace2image4rpu: " + str(task4trace2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4rpu : trace2image4rpu ; (Step 3A/33) try; task4trace2rpu-try-by-trace2image4rpu: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4rpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4rpu',
    #  'METHOD': 'trace2image4rpu'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        rpu2trace4xi ['CLASS']  = task4trace2rpu['CLASS']                                                      # RPUIZE RPU TRACE!
        rpu2trace4xi ['METHOD']  = task4trace2rpu['METHOD']
        rpu2trace4xi ['MESSAGE'] = task4trace2rpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; rpu2trace4xi-set-by-trace2image4rpu to: " + str(rpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4rpu : trace2image4rpu ; (Step 3B/33) set; rpu2trace4xi-set-by-trace2image4rpu to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4rpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4rpu',
        #                      'METHOD': 'trace2image4rpu'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; rpu-trace-failed-by-trace2image4rpu: " + str(task4trace2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE RPU TRACE
        # =================================
        class2print   = task4trace2rpu['CLASS']                                                                # RPUIZE RPU TRACE!
        method2print  = task4trace2rpu['METHOD']
        
        message2print = task4trace2rpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # RPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4rpu : trace2image4rpu ; (Step 2Z0/33) failed; trace2image4rpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4rpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4rpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-rpu-trace-failed-by-trace2image4rpu: " + str(task4trace2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4rpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4rpu"                
        message2print = "{(Step 3D/33) performed" + "; rpu-trace-performed-by-trace2image4rpu: " + str(task4trace2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4rpu : trace2image4rpu ; (Step 3D/33) performed; rpu-trace-performed-by-trace2image4rpu: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4rpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4rpu', 'METHOD': 'trace2image4rpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; rpu-trace-failed-by-trace2image4rpu: " + str(task4trace2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; rpu2trace4xi-returning-by-trace2image4rpu: " + str(rpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4rpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4rpu', 
    #  'METHOD': 'trace2image4rpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return rpu2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4rpu" usage = "
#
#   print('(Step 14A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF RPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4rpu()                                                                                                 
#       print('(Step 14B/700) OF RPU2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF RPU DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4rpu at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF RPU DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4rpu(task4set2imagecallback=None):

    '''
    registers callback function to rpu2image4xi
    '''
    global rpu2image4xi
    
    class2print = "set2imagecallback4rpu" 
    method2print = "set2imagecallback4rpu"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4rpu-started: " + str(set2imagecallback4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4rpu at 0x000000000390B748>
    
    try:
    
        rpu2image4xi ['CALLBACK'] = trace2image4rpu
        
        message2print = "{(Step 1B/70) set callback2rpu" + "; imagecallback4rpu-set-by-set2imagecallback4rpu: " + str(rpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4rpu at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-rpu2image4xi-returning-by-set2imagecallback4rpu: " + str(rpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4rpu at 0x0000000003864A48> 
        
        return rpu2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4rpu: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4rpu-returning-by-set2imagecallback4rpu: " + str(trace2image4rpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4rpu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4rpu" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF RPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4RPU ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4rpu()                                                                 
#       print('(Step 2B/700) OF RPU2IMAGE4XI5TEST - CREATED 2D IMAGE4RPU ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4RPU *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF RPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for RPU ', time4xi()) 
#   # 
#   try:
#       xi2rpu = RPU2Image4Xi()                            
#       response4create2Dimage = xi2rpu.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF RPU2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for RPU ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4rpu(task4imagecreate2rpu=None):

    '''
    creates 2D image for RPU
    '''
    
    global rpu2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4rpu" 
    method2print = "create2image4rpu"

    message2print = "{(Step 1A/70) started" + "; create2image4rpu-started: " + str(create2image4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4rpu at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2rpu-to-create2image4rpu: " + str(task4imagecreate2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    rpu2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2rpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO RPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2rpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2rpu-created-by-create2image4rpu to: " + str(task4imagecreate2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4rpu = trace2image4rpu                                                                 # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4rpu-created-by-create2image4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4rpu at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4rpu"                                                          # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4rpu"                                                          # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4rpu',
            #  'METHOD': 'create2image4rpu'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2rpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2rpu-created-by-create2image4rpu to: " + str(task4imagecreate2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4rpu at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4rpu',
            #  'METHOD': 'create2image4rpu'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2rpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2rpu-trying-by-create2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2rpu-got-by-create2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4rpu',
                #  'METHOD': 'create2context4rpu'},
                #  'FUNCTION2CALLBACK': <function trace2context4rpu at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2rpu-failed-by-create2image4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2rpu ['CONTEXT'] = context2rpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2rpu-trying-by-create2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2rpu-got-by-create2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_RPU',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2rpu-failed-by-create2image4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2rpu ['DEVICE'] = device2rpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2rpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2rpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2rpu-trying-by-create2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2rpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2rpu-got-by-create2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2rpu-failed-by-create2image4rpu: " + str(queue2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2rpu ['QUEUE'] = queue2rpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2rpu-trying-by-create2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2rpu-got-by-create2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2rpu-failed-by-create2image4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2rpu ['PIPE'] = pipe2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2rpu-trying-by-create2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2rpu-got-by-create2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4rpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4rpu',
                #  'METHOD': 'create2commandqueue4rpu'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4rpu at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4rpu', 
                #  'METHOD': 'create2context4rpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4rpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2rpu-failed-by-create2image4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2rpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2rpu-trying-by-create2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2rpu = get2bufferinfo4rpu()                                                                   # SYSTEM CREATES BUFFER FOR RPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2rpu-got-by-create2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4rpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4rpu',
                #                    'METHOD': 'create2buffer4rpu'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4rpu at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4rpu', 
                #  'METHOD': 'create2context4rpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4rpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2rpu-failed-by-create2image4rpu: " + str(bufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2rpu ['BUFFER'] = bufferinfo2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2rpu-trying-by-create2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2rpu-got-by-create2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4rpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4rpu',
                #                    'METHOD': 'create2event4rpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4rpu at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4rpu', 'METHOD': 'create2context4rpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4rpu at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2rpu-failed-by-create2image4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2rpu ['EVENT'] = eventinfo2rpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2rpu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2rpu-trying-by-create2image4rpu: " + str(flags2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2rpu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2rpu-set-by-create2image4rpu to: " + str(flags2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2rpu-failed-by-create2image4rpu: " + str(flags2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2rpu ['FLAGS'] = flags2rpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2rpu = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2rpu-trying-by-create2image4rpu: " + str(width2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2rpu = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2rpu = 2 * width2rpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2rpu-set-by-create2image4rpu to: " + str(width2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2rpu-failed-by-create2image4rpu: " + str(width2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2rpu ['WIDTH'] = width2rpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2rpu = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2rpu-trying-by-create2image4rpu: " + str(height2rpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2rpu = height2layer * quantity2total
                
                height2rpu = 2 * height2rpu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2rpu-set-by-create2image4rpu to: " + str(height2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2rpu-failed-by-create2image4rpu: " + str(height2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2rpu ['HEIGHT'] = height2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2rpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2rpu-trying-by-create2image4rpu: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2rpu = int(width2rpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2rpu-set-by-create2image4rpu to: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2rpu-failed-by-create2image4rpu: " + str(rowpitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2rpu ['ROWPITCH'] = rowpitch2rpu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2rpu = None
            size2rpu = int(width2rpu * height2rpu)                                                                  # CALCULATED MEMORY
            if size2rpu == 0:
                size2rpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2rpu-trying-by-create2image4rpu: " + str(pointer2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2rpu = Arena4Xi(size2rpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2rpu-got-by-create2image4rpu: " + str(arena2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2rpu.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4rpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2rpu.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4rpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2rpu:
                    size2rpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2rpu-set-by-create2buffer4rpu to: " + str(size2rpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2rpu = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2rpu-set-by-create2buffer4rpu to: " + str(pointer2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2rpu-failed-by-create2image4rpu: " + str(pointer2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2rpu ['POINTER'] = pointer2rpu                            
            task4imagecreate2rpu ['SIZE']    = size2rpu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2rpu = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2rpu-trying-by-create2image4rpu: " + str(format2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2rpu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2rpu-set-by-create2image4rpu to: " + str(format2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2rpu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2rpu-set-by-create2image4rpu to: " + str(format2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2rpu-failed-by-create2image4rpu: " + str(format2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2rpu ['FORMAT'] = format2rpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2rpu = 0
            message2print = "{(Step 19A/70) set" + "; depth2rpu-set-by-create2image4rpu to: " + str(depth2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2rpu ['DEPTH'] = depth2rpu
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2rpu-yielded-by-create2image4rpu: " + str(task4imagecreate2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2rpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4rpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR RPU! 
    # ---------------------------------------------------------------------------------------------    
    rpu2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; rpu2image4xi-initialized-by-create2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4rpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['DATA2CALLBACK'] = task4imagecreate2rpu['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4rpu to: " + str(rpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4rpu',
        #  'METHOD': 'create2image4rpu'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-rpu2image4xi-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2rpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4rpu to: " + str(rpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4rpu at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-rpu2image4xi-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['PLATFORM'] = task4imagecreate2rpu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4rpu to: " + str(rpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-rpu2image4xi-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2rpu-try-by-create2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['CONTEXT'] = task4imagecreate2rpu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4rpu to: " + str(rpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4rpu',
        #  'METHOD': 'create2context4rpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4rpu at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-rpu2image4xi-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['DEVICE'] = task4imagecreate2rpu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4rpu to: " + str(rpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-rpu2image4xi-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['CODE2ERROR'] = task4imagecreate2rpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4rpu to: " + str(rpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-rpu2image4xi-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['QUEUE'] = task4imagecreate2rpu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4rpu to: " + str(rpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-rpu2image4xi-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['PIPE'] = task4imagecreate2rpu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4rpu to: " + str(rpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-rpu2image4xi-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['COMMANDQUEUE']  = task4imagecreate2rpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4rpu to: " + str(rpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4rpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4rpu',
        #  'METHOD': 'create2commandqueue4rpu'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4rpu at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['BUFFER']  = task4imagecreate2rpu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4rpu to: " + str(rpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4rpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4rpu',
        #  'METHOD': 'create2buffer4rpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4rpu at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['EVENT']  = task4imagecreate2rpu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4rpu to: " + str(rpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4rpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4rpu',
        #                     'METHOD': 'create2event4rpu'},
        #  'FUNCTION2CALLBACK': <function trace2event4rpu at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['FLAGS']  = task4imagecreate2rpu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4rpu to: " + str(rpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        rpu2image4xi['WIDTH']  = task4imagecreate2rpu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4rpu to: " + str(rpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        rpu2image4xi['HEIGHT']  = task4imagecreate2rpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4rpu to: " + str(rpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['ROWPITCH']  = task4imagecreate2rpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4rpu to: " + str(rpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4rpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['POINTER']  = task4imagecreate2rpu['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4rpu to: " + str(rpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['FORMAT']  = task4imagecreate2rpu['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4rpu to: " + str(rpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['DEPTH']  = task4imagecreate2rpu['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4rpu to: " + str(rpu2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    rpu2image4xi ['ACK2RETURN']  = code2ack
    rpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; rpu2image4xi-returning-by-create2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return rpu2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4rpu" usage = "
#    
#   print('(Step 4B/33) OF RPU2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4RPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4RPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4RPU5TEST - TRYING TO INITIALIZE DEVICE4RPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device2rpu = init2rpu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4RPU5TEST - INITIALIZED DEVICE4RPU TO: ', device2rpu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4RPU5TEST *** FAILED TO INITIALIZE DEVICE4RPU', time4xi()) 
#   print('(Step 3A/33) OF RPU2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4RPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context2rpu = create2context4rpu()                                                                               
#       print('(Step 3B/33) OF RPU2IMAGE4XI5TEST - CREATED CONTEXT4RPU: ', context2rpu)                                  
#   except:
#       print('(Step 3Z/33) OF RPU2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4RPU', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2rpu  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4RPU ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4rpu(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF RPU2IMAGE4XI5TEST - SET 2D IMAGE4RPU ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4RPU *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4rpu(task4imageset2rpu=None):

    '''
    sets 2D images for RPU
    '''
    global rpu2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4rpu"
    method2print = "set2image4rpu"

    message2print = "{(Step 1A/33) started" + "; set2image4rpu-started: " + str(set2image4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4rpu at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2rpu-to-set2image4rpu: " + str(task4imageset2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4rpu',
    #              'METHOD': 'create2context4rpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4rpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4rpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4rpu',
    #              'METHOD': 'create2commandqueue4rpu'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4rpu at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4rpu',
    #              'METHOD': 'create2context4rpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4rpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if rpu2image4xi is None:
            rpu2image4xi = create2image4rpu()                                                                              # INITIALIZE IMAGE FOR RPU 
            message2print = "{(Step 1C/70) set" + "; rpu2image4xi-set-by-set2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4rpu-skipped-by-set2image4rpu: " + str(create2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4rpu at 0x0000000003A627C8>;                                  
    except:
        try:
            rpu2image4xi = create2image4rpu()
            message2print = "{(Step 1E/70) set" + "; rpu2image4xi-set-by-set2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            rpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; rpu2image4xi-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['DATA2CALLBACK'] = task4imageset2rpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4rpu to: " + str(rpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-rpu2image4xi-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['FUNCTION2CALLBACK'] = task4imageset2rpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4rpu to: " + str(rpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-rpu2image4xi-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['PLATFORM'] = task4imageset2rpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4rpu to: " + str(rpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-rpu2image4xi-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2rpu-try-by-set2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['CONTEXT'] = task4imageset2rpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4rpu to: " + str(rpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4rpu',
        #                    'METHOD': 'create2context4rpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4rpu at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-rpu2image4xi-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['DEVICE'] = task4imageset2rpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4rpu to: " + str(rpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_RPU',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-rpu2image4xi-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['CODE2ERROR'] = task4imageset2rpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4rpu to: " + str(rpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-rpu2image4xi-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['QUEUE'] = task4imageset2rpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4rpu to: " + str(rpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-rpu2image4xi-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['PIPE'] = task4imageset2rpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4rpu to: " + str(rpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-rpu2image4xi-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['COMMANDQUEUE']  = task4imageset2rpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4rpu to: " + str(rpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['BUFFER']  = task4imageset2rpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4rpu to: " + str(rpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4rpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['EVENT']  = task4imageset2rpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4rpu to: " + str(rpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['FLAGS']  = task4imageset2rpu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4rpu to: " + str(rpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        rpu2image4xi['WIDTH']  = task4imageset2rpu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4rpu to: " + str(rpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4rpu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        rpu2image4xi['HEIGHT']  = task4imageset2rpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4rpu to: " + str(rpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['ROWPITCH']  = task4imageset2rpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4rpu to: " + str(rpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['POINTER']  = task4imageset2rpu['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4rpu to: " + str(rpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4rpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        rpu2image4xi['FORMAT']  = task4imageset2rpu['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4rpu to: " + str(rpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    rpu2image4xi ['ACK2RETURN']  = code2ack
    rpu2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; rpu2image4xi-returning-by-set2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return rpu2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4rpu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4RPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4RPU ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4rpu(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4RPU : ', response4copy2image)
#       queue2copy4rpu      = response4copy2image['QUEUE']
#       channel2copypublish4rpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4rpu = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4RPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4RPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4RPU ', time4xi())     
#   try:                               
#       message2copyimagesend4rpu = "<copy4xi>copy2copy4rpu</copy4xi>"
#       channel2copypublish4rpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4rpu.send(message2copyimagesend4rpu)
#       print("(Step 17B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2copypublish4rpu, " MESSAGE2SEND4RPU: " , message2copyimagesend4rpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4RPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4RPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2copyimage4rpu(queue2copy4rpu, channel2copyconsume4rpu, channel2copypublish4rpu)
#       print("(Step 18D/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", manoeuvre2copyimage4rpu)
#   except:
#       print("(Step 18Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4RPU ', time4xi())     
#   try:
#       channel2copyconsume4rpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4rpu = channel2copyconsume4rpu.recv()
#       print("(Step 19B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2copyconsume4rpu, " CONTROL & COMMAND: ", message2copyimagereceive4rpu)                                                                                                  
#       # <roger4xi>roger2copy4rpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4RPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4RPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4RPU ', time4xi())     
#   try:                               
#       message2copyimagesend4rpu = "<copy4xi>copy2copy4rpu</copy4xi>"
#       channel2copypublish4rpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4rpu.send(message2copyimagesend4rpu)
#       print("(Step 20B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2copypublish4rpu, " MESSAGE2SEND4RPU: " , message2copyimagesend4rpu)                                                                                                
#       # <copy4xi>copy2copy4rpu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4RPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4RPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2copyimage4rpu(queue2copy4rpu, channel2copyconsume4rpu, channel2copypublish4rpu)
#       print("(Step 21B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", manoeuvre2copyimage4rpu)
#   except:
#       print("(Step 21Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4RPU ', time4xi())     
#   try:
#       channel2copyconsume4rpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4rpu = channel2copyconsume4rpu.recv()
#       print("(Step 22B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2copyconsume4rpu, " CONTROL & COMMAND: ", message2copyimagereceive4rpu)                                                                                                  
#       # <copy4xi>copy2copy4rpu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4RPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4RPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2copyimage4rpu(queue2copy4rpu, channel2copyconsume4rpu, channel2copypublish4rpu)
#       print("(Step 25B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", manoeuvre2copyimage4rpu)
#   except:
#       print("(Step 25Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4rpu = channel2copyconsume4rpu.recv()
#       print("(Step 26D/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2copyconsume4rpu, " CONTROL & COMMAND: ", message2copyimagereceive4rpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4rpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4RPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2copyimage4rpu(queue2copy4rpu, channel2copyconsume4rpu, channel2copypublish4rpu)
#       print("(Step 27B/700) OF RPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4RPU ", response2manoeuvre4rpu)
#   except:
#       print("(Step 27Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4rpu(task4imagecopy2rpu=None):

    '''
    copies 2D images for RPU
    '''
    global rpu2image4xi                                                                                             # 2D-IMAGES FOR RPU  
    global control2copyimage4rpu                                                                                    # COMMAND AND CONTROL FOR RPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4rpu"
    method2print = "copy2image4rpu"

    message2print = "{(Step 0A/33) started" + "; copy2image4rpu-started: " + str(copy2image4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4rpu at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2rpu-to-copy2image4rpu: " + str(task4imagecopy2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE RPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if rpu2image4xi is None:
            rpu2image4xi = create2image4rpu()                                                                       # INITIALIZE IMAGE FOR RPU 
            message2print = "{(Step 0C/70) created" + "; rpu2image4xi-created-by-copy2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4rpu-skipped-by-copy2image4rpu: " + str(copy2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4rpu at 0x00000000038538C8>                                 
    except:
        try:
            rpu2image4xi = create2image4rpu()
            message2print = "{(Step 0E/70) set" + "; rpu2image4xi-set-by-copy2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            rpu2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; rpu2image4xi-yielded-by-copy2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2rpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2rpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2rpu-created-by-copy2image4rpu to: " + str(task4imagecopy2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4rpu = trace2image4rpu                                                                 # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4rpu-created-by-copy2image4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4rpu at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4rpu"                                                            # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4rpu"                                                            # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4rpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4rpu',
            #  'METHOD': 'copy2image4rpu'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2rpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2rpu-created-by-copy2image4rpu to: " + str(task4imagecopy2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4rpu at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4rpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4rpu',
            #  'METHOD': 'copy2image4rpu'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2rpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2rpu-trying-by-copy2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2rpu-got-by-copy2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2rpu-failed-by-copy2image4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2rpu ['CONTEXT'] = context2rpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2rpu-trying-by-copy2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2rpu-got-by-copy2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2rpu-failed-by-copy2image4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2rpu ['DEVICE'] = device2rpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2rpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2rpu-trying-by-copy2image4rpu: " + str(imagesourceinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2rpu-got-by-copy2image4rpu: " + str(imagesourceinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2rpu-failed-by-copy2image4rpu: " + str(imagesourceinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2rpu ['IMAGESOURCE'] = imagesourceinfo2rpu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2rpu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2rpu-trying-by-copy2image4rpu: " + str(imagedestinationinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2rpu-got-by-copy2image4rpu: " + str(imagedestinationinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2rpu-failed-by-copy2image4rpu: " + str(imagedestinationinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2rpu ['IMAGEDESTINATION'] = imagedestinationinfo2rpu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY RPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2rpu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2rpu-trying-by-copy2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2rpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2rpu-got-by-copy2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2rpu-failed-by-copy2image4rpu: " + str(queue2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2rpu ['QUEUE'] = queue2rpu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY RPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2rpu-trying-by-copy2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2rpu-got-by-copy2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2rpu-failed-by-copy2image4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2rpu ['PIPE'] = pipe2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2rpu-trying-by-copy2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2rpu-got-by-copy2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4rpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4rpu', 
                #  'METHOD': 'create2commandqueue4rpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4rpu at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2rpu-failed-by-copy2image4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY RPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2rpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2rpu-trying-by-copy2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2rpu = get2bufferinfo4rpu()                                                               # SYSTEM CREATES BUFFER FOR RPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2rpu-got-by-copy2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2rpu-failed-by-copy2image4rpu: " + str(bufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2rpu ['BUFFER'] = bufferinfo2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2rpu-trying-by-copy2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2rpu-got-by-copy2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4rpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4rpu',
                #  'METHOD': 'create2event4rpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4rpu at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2rpu-failed-by-copy2image4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2rpu ['EVENT'] = eventinfo2rpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2rpu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2rpu-trying-by-copy2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2rpu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2rpu-set-by-copy2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2rpu-failed-by-copy2image4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2rpu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-copy2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2rpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2rpu-set-by-copy2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-copy2image4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2rpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2rpu-trying-by-copy2image4rpu: " + str(origin4source2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2rpu['X'] = 0 
                origin4source2rpu['Y'] = 0  
                origin4source2rpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2rpu-set-by-copy2image4rpu to: " + str(origin4source2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2rpu-failed-by-copy2image4rpu: " + str(origin4source2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2rpu ['SOURCEORIGIN'] = origin4source2rpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2rpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2rpu-trying-by-copy2image4rpu: " + str(origin4destination2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2rpu['X'] = 100 
                origin4destination2rpu['Y'] = 100  
                origin4destination2rpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2rpu-set-by-copy2image4rpu to: " + str(origin4destination2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2rpu-failed-by-copy2image4rpu: " + str(origin4destination2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2rpu ['DESTINATIONORIGIN'] = origin4destination2rpu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2rpu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2rpu-trying-by-copy2image4rpu: " + str(region4copy2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2rpu['X'] = 10 
                region4copy2rpu['Y'] = 10  
                region4copy2rpu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2rpu-set-by-copy2image4rpu to: " + str(region4copy2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2rpu-failed-by-copy2image4rpu: " + str(region4copy2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2rpu ['REGION'] = region4copy2rpu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2rpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4rpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4rpu = trace2image4rpu                                                            # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4rpu-created-by-copy2image4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4rpu"                                                        # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4rpu"                                                        # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2rpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2rpu-created-by-copy2image4rpu to: " + str(task4imagecopy2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2rpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2rpu-trying-by-copy2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2rpu-got-by-copy2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2rpu-failed-by-copy2image4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2rpu ['CONTEXT'] = context2rpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2rpu-trying-by-copy2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2rpu-got-by-copy2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2rpu-failed-by-copy2image4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2rpu ['DEVICE'] = device2rpu
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2rpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2rpu-trying-by-copy2image4rpu: " + str(sourceimageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2rpu-got-by-copy2image4rpu: " + str(sourceimageinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2rpu-failed-by-copy2image4rpu: " + str(sourceimageinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2rpu ['IMAGESOURCE'] = sourceimageinfo2rpu         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2rpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2rpu-trying-by-copy2image4rpu: " + str(destinationimageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2rpu-got-by-copy2image4rpu: " + str(destinationimageinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2rpu-failed-by-copy2image4rpu: " + str(destinationimageinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2rpu ['IMAGEDESTINATION'] = destinationimageinfo2rpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY RPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2rpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2rpu-trying-by-copy2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2rpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2rpu-got-by-copy2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2rpu-failed-by-copy2image4rpu: " + str(queue2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2rpu ['QUEUE'] = queue2rpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY RPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2rpu-trying-by-copy2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2rpu-got-by-copy2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2rpu-failed-by-copy2image4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2rpu ['PIPE'] = pipe2rpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2rpu-trying-by-copy2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2rpu-got-by-copy2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2rpu-failed-by-copy2image4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY RPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2rpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2rpu-trying-by-copy2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2rpu = get2bufferinfo4rpu()                                                           # SYSTEM CREATES BUFFER FOR RPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2rpu-got-by-copy2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2rpu-failed-by-copy2image4rpu: " + str(bufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2rpu ['BUFFER'] = bufferinfo2rpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2rpu-trying-by-copy2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2rpu-got-by-copy2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2rpu-failed-by-copy2image4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2rpu ['EVENT'] = eventinfo2rpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2rpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2rpu-trying-by-copy2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2rpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2rpu-set-by-copy2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2rpu-failed-by-copy2image4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2rpu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-copy2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2rpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2rpu-set-by-copy2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-copy2image4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2rpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2rpu-trying-by-copy2image4rpu: " + str(source4origin2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2rpu['X'] = 0 
                source4origin2rpu['Y'] = 0  
                source4origin2rpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2rpu-set-by-copy2image4rpu to: " + str(source4origin2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2rpu-failed-by-copy2image4rpu: " + str(source4origin2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2rpu ['SOURCEORIGIN'] = source4origin2rpu
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2rpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2rpu-trying-by-copy2image4rpu: " + str(destination4origin2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2rpu['X'] = 100 
                destination4origin2rpu['Y'] = 100  
                destination4origin2rpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2rpu-set-by-copy2image4rpu to: " + str(destination4origin2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2rpu-failed-by-copy2image4rpu: " + str(destination4origin2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2rpu ['DESTINATIONSOURCEORIGIN'] = destination4origin2rpu                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2rpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2rpu-trying-by-copy2image4rpu: " + str(region2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2rpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2rpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2rpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2rpu-set-by-copy2image4rpu to: " + str(region2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2rpu-failed-by-copy2image4rpu: " + str(region2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2rpu ['REGION'] = region2rpu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2rpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4rpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2rpu-yielded-by-copy2image4rpu: " + str(task4imagecopy2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4rpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2rpu = False
    channels2rpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2rpu-trying-by-copy2image4rpu: " + str(channels2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2rpu, publisherchannel2rpu = task4imagecopy2rpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2rpu-got-by-copy2image4rpu: " + str(consumerchannel2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2rpu-got-by-copy2image4rpu: " + str(publisherchannel2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2rpu = True
        
        control2copyimage4rpu['CONSUMERCHANNEL']  = consumerchannel2rpu
        control2copyimage4rpu['PUBLISHERCHANNEL'] = publisherchannel2rpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2rpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2rpu = False
    queue2rpu = None
    message2print = "{(Step 41A/70) trying" + "queue2rpu-trying-by-copy2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2rpu = task4imagecopy2rpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2rpu-got-by-copy2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2rpu = True
        control2copyimage4rpu['QUEUE']  = queue2rpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2rpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4rpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4rpu-trying-by-copy2image4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4rpu = DummyProcess4Xi(target=manoeuvre2copyimage4rpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2rpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2rpu, publisherchannel2rpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4rpu-created-by-copy2image4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4rpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4rpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4rpu-returning-by-copy2image4rpu: " + str(control2copyimage4rpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4rpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4rpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4RPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4RPU ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4rpu(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4RPU : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4rpu      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4rpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4rpu = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4RPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4RPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4RPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4rpu = "<copy4xi>copy2copyimagetobuffer4rpu</copy4xi>"
#       channel2copyimagetobufferpublish4rpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4rpu.send(message2copyimagetobuffersend4rpu)
#       print("(Step 61B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2copyimagetobufferpublish4rpu, " MESSAGE2SEND4RPU: " , message2copyimagetobuffersend4rpu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4RPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4RPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4rpu = manoeuvre2copyimagetobuffer4rpu(queue2copyimagetobuffer4rpu, channel2copyimagetobufferconsume4rpu, channel2copyimagetobufferpublish4rpu)
#       print("(Step 62B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", manoeuvre2copyimagetobuffer4rpu)
#   except:
#       print("(Step 62Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4RPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4rpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4rpu = channel2copyimagetobufferconsume4rpu.recv()
#       print("(Step 63B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2copyimagetobufferconsume4rpu, " CONTROL & COMMAND: ", message2copyimagereceive4rpu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4RPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4RPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4RPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4rpu = "<copy4xi>copy2copyimagetobuffer4rpu</copy4xi>"
#       channel2copyimagetobufferpublish4rpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4rpu.send(message2copyimagetobuffersend4rpu)
#       print("(Step 64B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2copyimagetobufferpublish4rpu, " MESSAGE2SEND4RPU: " , message2copyimagetobuffersend4rpu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4RPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4RPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4rpu = manoeuvre2copyimagetobuffer4rpu(queue2copyimagetobuffer4rpu, channel2copyimagetobufferconsume4rpu, channel2copyimagetobufferpublish4rpu)
#       print("(Step 65B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", manoeuvre2copyimagetobuffer4rpu)
#   except:
#       print("(Step 65Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4RPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4rpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4rpu = channel2copyimagetobufferconsume4rpu.recv()
#       print("(Step 66B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2copyimagetobufferconsume4rpu, " CONTROL & COMMAND: ", message2copyimagereceive4rpu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4RPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4RPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4rpu = manoeuvre2copyimagetobuffer4rpu(queue2copyimagetobuffer4rpu, channel2copyimagetobufferconsume4rpu, channel2copyimagetobufferpublish4rpu)
#       print("(Step 68B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", manoeuvre2copyimagetobuffer4rpu)
#   except:
#       print("(Step 68Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4rpu = channel2copyimagetobufferconsume4rpu.recv()
#       print("(Step 69D/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2copyimagetobufferconsume4rpu, " CONTROL & COMMAND: ", message2copyimagereceive4rpu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4RPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4rpu = manoeuvre2copyimagetobuffer4rpu(queue2copyimagetobuffer4rpu, channel2copyimagetobufferconsume4rpu, channel2copyimagetobufferpublish4rpu)
#       print("(Step 70B/700) OF RPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4RPU ", response2manoeuvreimagecopytobuffer4rpu)
#   except:
#       print("(Step 70Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4rpu(task4copyimagetobuffer2rpu=None):

    '''
    copies 2D images to buffer for RPU
    '''
    global rpu2image4xi                                                                                             # 2D-IMAGES FOR RPU  
    global rpu2buffer4xi                                                                                            # BUFFER FOR RPU
    
    global control2copyimagetobuffer4rpu                                                                            # COMMAND AND CONTROL FOR RPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4rpu"
    method2print = "copy2imagetobuffer4rpu"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4rpu-started: " + str(copy2imagetobuffer4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4rpu at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2rpu-to-copy2imagetobuffer4rpu: " + str(task4copyimagetobuffer2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE RPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if rpu2image4xi is None:
            rpu2image4xi = create2image4rpu()                                                                              # INITIALIZE IMAGE FOR RPU 
            message2print = "{(Step 1C/70) created" + "; rpu2image4xi-created-by-copy2imagetobuffer4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4rpu-skipped-by-copy2imagetobuffer4rpu: " + str(copy2imagetobuffer4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4rpu-skipped-by-copy2imagetobuffer4rpu                                 
    except:
        try:
            rpu2image4xi = create2image4rpu()
            message2print = "{(Step 1E/70) set" + "; rpu2image4xi-set-by-copy2imagetobuffer4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            rpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; rpu2image4xi-yielded-by-copy2imagetobuffer4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE RPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if rpu2buffer4xi is None:
            rpu2buffer4xi = create2buffer4rpu()                                                                     # INITIALIZE BUFFER FOR RPU 
            message2print = "{(Step 1F/70) created" + "; rpu2buffer4xi-created-by-copy2imagetobuffer4rpu to: " + str(rpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4rpu-skipped-by-copy2imagetobuffer4rpu: " + str(copy2imagetobuffer4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            rpu2buffer4xi = create2buffer4rpu()
            message2print = "{(Step 1H/70) set" + "; rpu2buffer4xi-set-by-copy2imagetobuffer4rpu to: " + str(rpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4rpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4rpu',
            #   'METHOD': 'create2buffer4rpu'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4rpu at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            rpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; rpu2buffer4xi-yielded-by-copy2imagetobuffer4rpu: " + str(rpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2rpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2rpu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2rpu-created-by-copy2imagetobuffer4rpu to: " + str(task4copyimagetobuffer2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4rpu = trace2image4rpu                                                                 # CALLBACK FUNCTION FOR RPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4rpu-created-by-copy2imagetobuffer4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4rpu at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4rpu"                                                            # MODULE OF RPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4rpu"                                                            # FUNCTION OF RPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4rpu-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4rpu', 
        #  'METHOD': 'copy2imagetobuffer4rpu'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2rpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2rpu-created-by-copy2imagetobuffer4rpu to: " + str(task4copyimagetobuffer2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4rpu at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4rpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4rpu', 
        #  'METHOD': 'copy2imagetobuffer4rpu'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2rpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2rpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2rpu-trying-by-copy2imagetobuffer4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2rpu = get2context4rpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2rpu-got-by-copy2imagetobuffer4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4rpu', 
            #  'METHOD': 'create2context4rpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4rpu at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2rpu-failed-by-copy2imagetobuffer4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2rpu ['CONTEXT'] = context2rpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2rpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2rpu-trying-by-copy2imagetobuffer4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2rpu = get2rpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2rpu-got-by-copy2imagetobuffer4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_RPU', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2rpu-failed-by-copy2imagetobuffer4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2rpu ['DEVICE'] = device2rpu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2rpu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2rpu-trying-by-copy2imagetobuffer4rpu: " + str(imagesourceinfo2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2rpu = get2imageinfo4rpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2rpu-got-by-copy2imagetobuffer4rpu: " + str(imagesourceinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2rpu-failed-by-copy2imagetobuffer4rpu: " + str(imagesourceinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2rpu ['IMAGESOURCE'] = imagesourceinfo2rpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2rpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2rpu-trying-by-copy2imagetobuffer4rpu: " + str(bufferdestinationinfo2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2rpu = get2bufferinfo4rpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2rpu-got-by-copy2imagetobuffer4rpu: " + str(bufferdestinationinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4rpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4rpu', 
            #  'METHOD': 'create2buffer4rpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4rpu at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2rpu-failed-by-copy2imagetobuffer4rpu: " + str(bufferdestinationinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2rpu ['BUFFERDESTINATION'] = bufferdestinationinfo2rpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY RPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4rpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4rpu-trying-by-copy2imagetobuffer4rpu: " + str(queue2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4rpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4rpu-got-by-copy2imagetobuffer4rpu: " + str(queue2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4rpu-failed-by-copy2imagetobuffer4rpu: " + str(queue2copyimagetobuffer4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2rpu ['QUEUE'] = queue2copyimagetobuffer4rpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY RPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2rpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2rpu-trying-by-copy2imagetobuffer4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2rpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2rpu-got-by-copy2imagetobuffer4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2rpu-failed-by-copy2imagetobuffer4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2rpu ['PIPE'] = pipe2rpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2rpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2rpu-trying-by-copy2imagetobuffer4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2rpu-got-by-copy2imagetobuffer4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4rpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4rpu', 
            #  'METHOD': 'create2commandqueue4rpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4rpu at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2rpu-failed-by-copy2imagetobuffer4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2rpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2rpu-trying-by-copy2imagetobuffer4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2rpu = get2eventinfo4rpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2rpu-got-by-copy2imagetobuffer4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4rpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4rpu', 
            #  'METHOD': 'create2event4rpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4rpu at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2rpu-failed-by-copy2imagetobuffer4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2rpu ['EVENT'] = eventinfo2rpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2rpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2rpu-trying-by-copy2imagetobuffer4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2rpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2rpu-set-by-copy2imagetobuffer4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2rpu-failed-by-copy2imagetobuffer4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2rpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-copy2imagetobuffer4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2rpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2rpu-set-by-copy2imagetobuffer4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-copy2imagetobuffer4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2rpu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2rpu-trying-by-copy2imagetobuffer4rpu: " + str(origin4source2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2rpu['X'] = 0 
            origin4source2rpu['Y'] = 0  
            origin4source2rpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2rpu-set-by-copy2imagetobuffer4rpu to: " + str(origin4source2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2rpu-failed-by-copy2imagetobuffer4rpu: " + str(origin4source2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2rpu ['SOURCEORIGIN'] = origin4source2rpu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2rpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2rpu-trying-by-copy2imagetobuffer4rpu: " + str(region4copy2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2rpu['X'] = 10 
            region4copy2rpu['Y'] = 10  
            region4copy2rpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2rpu-set-by-copy2imagetobuffer4rpu to: " + str(region4copy2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2rpu-failed-by-copy2imagetobuffer4rpu: " + str(region4copy2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2rpu ['REGION'] = region4copy2rpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2rpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2rpu-set-by-copy2imagetobuffer4rpu to: " + str(offset4copy2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2rpu ['OFFSET'] = offset4copy2rpu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2rpu.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4rpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4rpu = trace2image4rpu                                                             # CALLBACK FUNCTION FOR RPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4rpu-created-by-copy2imagetobuffer4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4rpu"                                                        # MODULE OF RPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4rpu"                                                        # FUNCTION OF RPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2rpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2rpu-created-by-copy2imagetobuffer4rpu to: " + str(task4copyimagetobuffer2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2rpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2rpu = {}
        message2print = "{(Step 23A/70) trying" + "; context2rpu-trying-by-copy2imagetobuffer4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2rpu = get2context4rpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2rpu-got-by-copy2imagetobuffer4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2rpu-failed-by-copy2imagetobuffer4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2rpu ['CONTEXT'] = context2rpu           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2rpu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2rpu-trying-by-copy2imagetobuffer4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2rpu = get2rpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2rpu-got-by-copy2imagetobuffer4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2rpu-failed-by-copy2imagetobuffer4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2rpu ['DEVICE'] = device2rpu
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2rpu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2rpu-trying-by-copy2imagetobuffer4rpu: " + str(sourceimageinfo2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2rpu = get2imageinfo4rpu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2rpu-got-by-copy2imagetobuffer4rpu: " + str(sourceimageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2rpu-failed-by-copy2imagetobuffer4rpu: " + str(sourceimageinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2rpu ['IMAGESOURCE'] = sourceimageinfo2rpu         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2rpu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2rpu-trying-by-copy2imagetobuffer4rpu: " + str(destinationbufferinfo2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2rpu = get2bufferinfo4rpu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2rpu-got-by-copy2imagetobuffer4rpu: " + str(destinationbufferinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2rpu-failed-by-copy2imagetobuffer4rpu: " + str(destinationbufferinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2rpu ['BUFFERDESTINATION'] = destinationbufferinfo2rpu    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY RPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4rpu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4rpu-trying-by-copy2imagetobuffer4rpu: " + str(queue2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4rpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4rpu-got-by-copy2imagetobuffer4rpu: " + str(queue2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4rpu-failed-by-copy2imagetobuffer4rpu: " + str(queue2copyimagetobuffer4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2rpu ['QUEUE'] = queue2copyimagetobuffer4rpu
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY RPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2rpu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2rpu-trying-by-copy2imagetobuffer4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2rpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2rpu-got-by-copy2imagetobuffer4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2rpu-failed-by-copy2imagetobuffer4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2rpu ['PIPE'] = pipe2rpu 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2rpu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2rpu-trying-by-copy2imagetobuffer4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2rpu-got-by-copy2imagetobuffer4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2rpu-failed-by-copy2imagetobuffer4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2rpu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2rpu-trying-by-copy2imagetobuffer4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2rpu = get2eventinfo4rpu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2rpu-got-by-copy2imagetobuffer4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2rpu-failed-by-copy2imagetobuffer4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2rpu ['EVENT'] = eventinfo2rpu 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2rpu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2rpu-trying-by-copy2imagetobuffer4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2rpu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2rpu-set-by-copy2imagetobuffer4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2rpu-failed-by-copy2imagetobuffer4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2rpu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-copy2imagetobuffer4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2rpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2rpu-set-by-copy2imagetobuffer4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-copy2imagetobuffer4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2rpu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2rpu-trying-by-copy2imagetobuffer4rpu: " + str(source4origin2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2rpu['X'] = 0 
            source4origin2rpu['Y'] = 0  
            source4origin2rpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2rpu-set-by-copy2imagetobuffer4rpu to: " + str(source4origin2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2rpu-failed-by-copy2imagetobuffer4rpu: " + str(source4origin2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2rpu ['SOURCEORIGIN'] = source4origin2rpu
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2rpu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2rpu-trying-by-copy2imagetobuffer4rpu: " + str(region2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2rpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2rpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2rpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2rpu-set-by-copy2imagetobuffer4rpu to: " + str(region2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2rpu-failed-by-copy2imagetobuffer4rpu: " + str(region2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2rpu ['REGION'] = region2rpu
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2rpu = 0
        message2print = "{(Step 37A/70) set" + "; offset2rpu-set-by-copy2imagetobuffer4rpu to: " + str(offset2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2rpu ['OFFSET'] = offset2rpu        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2rpu.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4rpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4rpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2rpu = False
    channels2rpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2rpu-trying-by-copy2imagetobuffer4rpu: " + str(channels2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4rpu, publisherchannel2copyimagetobuffer4rpu = task4copyimagetobuffer2rpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4rpu-got-by-copy2imagetobuffer4rpu: " + str(consumerchannel2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4rpu-got-by-copy2imagetobuffer4rpu: " + str(publisherchannel2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2rpu = True
        
        control2copyimagetobuffer4rpu['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4rpu
        
        
        control2copyimagetobuffer4rpu['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4rpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2rpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4rpu = False
    queue2copyimagetobuffer4rpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4rpu-trying-by-copy2imagetobuffer4rpu: " + str(queue2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4rpu = task4copyimagetobuffer2rpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4rpu-got-by-copy2imagetobuffer4rpu: " + str(queue2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4rpu = True
        control2copyimagetobuffer4rpu['QUEUE']  = queue2copyimagetobuffer4rpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4rpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4rpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4rpu-trying-by-copy2imagetobuffer4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4rpu = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4rpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4rpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4rpu, publisherchannel2copyimagetobuffer4rpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4rpu-created-by-copy2imagetobuffer4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4rpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4rpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4rpu-returning-by-copy2imagetobuffer4rpu: " + str(control2copyimagetobuffer4rpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4rpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4rpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4RPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4RPU ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4rpu(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4RPU : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4rpu      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4rpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4rpu = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4RPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4RPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4RPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4rpu = "<copy4xi>copy2copybuffertoimage4rpu</copy4xi>"
#       channel2copybuffertoimagepublish4rpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4rpu.send(message2copybuffertoimagesend4rpu)
#       print("(Step 81B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2copybuffertoimagepublish4rpu, " MESSAGE2SEND4RPU: " , message2copybuffertoimagesend4rpu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4RPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4RPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4rpu = manoeuvre2copybuffertoimage4rpu(queue2copybuffertoimage4rpu, channel2copybuffertoimageconsume4rpu, channel2copybuffertoimagepublish4rpu)
#       print("(Step 82D/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", manoeuvre2copybuffertoimage4rpu)
#   except:
#       print("(Step 82Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4RPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4rpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4rpu = channel2copybuffertoimageconsume4rpu.recv()
#       print("(Step 83B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2copybuffertoimageconsume4rpu, " CONTROL & COMMAND: ", message2copyimagereceive4rpu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4RPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4RPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4RPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4rpu = "<copy4xi>copy2copybuffertoimage4rpu</copy4xi>"
#       channel2copybuffertoimagepublish4rpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4rpu.send(message2copybuffertoimagesend4rpu)
#       print("(Step 84B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2copybuffertoimagepublish4rpu, " MESSAGE2SEND4RPU: " , message2copybuffertoimagesend4rpu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4RPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4RPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4rpu = manoeuvre2copybuffertoimage4rpu(queue2copybuffertoimage4rpu, channel2copybuffertoimageconsume4rpu, channel2copybuffertoimagepublish4rpu)
#       print("(Step 85B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", manoeuvre2copybuffertoimage4rpu)
#   except:
#       print("(Step 85Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4RPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4rpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4rpu = channel2copybuffertoimageconsume4rpu.recv()
#       print("(Step 86B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2copybuffertoimageconsume4rpu, " CONTROL & COMMAND: ", message2copyimagereceive4rpu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4RPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4RPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4rpu = manoeuvre2copybuffertoimage4rpu(queue2copybuffertoimage4rpu, channel2copybuffertoimageconsume4rpu, channel2copybuffertoimagepublish4rpu)
#       print("(Step 88B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", manoeuvre2copybuffertoimage4rpu)
#   except:
#       print("(Step 88Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4rpu = channel2copybuffertoimageconsume4rpu.recv()
#       print("(Step 89D/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2copybuffertoimageconsume4rpu, " CONTROL & COMMAND: ", message2copyimagereceive4rpu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4RPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4rpu = manoeuvre2copybuffertoimage4rpu(queue2copybuffertoimage4rpu, channel2copybuffertoimageconsume4rpu, channel2copybuffertoimagepublish4rpu)
#       print("(Step 90B/700) OF RPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4RPU ", response2manoeuvreimagecopytobuffer4rpu)
#   except:
#       print("(Step 90Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4RPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4rpu(task4copybuffertoimage2rpu=None):

    '''
    copies buffer to image for RPU
    '''
    global rpu2image4xi                                                                                             # 2D-IMAGES FOR RPU  
    global rpu2buffer4xi                                                                                            # BUFFER FOR RPU
    
    global control2copybuffertoimage4rpu                                                                            # COMMAND AND CONTROL FOR RPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4rpu"
    method2print = "copy2buffertoimage4rpu"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4rpu-started: " + str(copy2buffertoimage4rpu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4rpu at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2rpu-to-copy2buffertoimage4rpu: " + str(task4copybuffertoimage2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE RPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if rpu2image4xi is None:
            rpu2image4xi = create2image4rpu()                                                                              # INITIALIZE IMAGE FOR RPU 
            message2print = "{(Step 1C/70) created" + "; rpu2image4xi-created-by-copy2buffertoimage4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4rpu-skipped-by-copy2buffertoimage4rpu: " + str(copy2buffertoimage4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            rpu2image4xi = create2image4rpu()
            message2print = "{(Step 1E/70) set" + "; rpu2image4xi-set-by-copy2buffertoimage4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            rpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; rpu2image4xi-yielded-by-copy2buffertoimage4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE RPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if rpu2buffer4xi is None:
            rpu2buffer4xi = create2buffer4rpu()                                                                     # INITIALIZE BUFFER FOR RPU 
            message2print = "{(Step 1F/70) created" + "; rpu2buffer4xi-created-by-copy2buffertoimage4rpu to: " + str(rpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4rpu-skipped-by-copy2buffertoimage4rpu: " + str(copy2buffertoimage4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            rpu2buffer4xi = create2buffer4rpu()
            message2print = "{(Step 1H/70) set" + "; rpu2buffer4xi-set-by-copy2buffertoimage4rpu to: " + str(rpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            rpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; rpu2buffer4xi-yielded-by-copy2buffertoimage4rpu: " + str(rpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2rpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2rpu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2rpu-created-by-copy2buffertoimage4rpu to: " + str(task4copybuffertoimage2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4rpu = trace2image4rpu                                                                 # CALLBACK FUNCTION FOR RPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4rpu-created-by-copy2buffertoimage4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4rpu at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4rpu"                                                     # MODULE OF RPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4rpu"                                                     # FUNCTION OF RPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4rpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4rpu', 
        #  'METHOD': 'copy2buffertoimage4rpu'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2rpu['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2rpu-created-by-copy2buffertoimage4rpu to: " + str(task4copybuffertoimage2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4rpu at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4rpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4rpu', 
        #  'METHOD': 'copy2buffertoimage4rpu'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2rpu['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2rpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2rpu-trying-by-copy2buffertoimage4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2rpu = get2context4rpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2rpu-got-by-copy2buffertoimage4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2rpu-failed-by-copy2buffertoimage4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2rpu ['CONTEXT'] = context2rpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2rpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2rpu-trying-by-copy2buffertoimage4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2rpu = get2rpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2rpu-got-by-copy2buffertoimage4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2rpu-failed-by-copy2buffertoimage4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2rpu ['DEVICE'] = device2rpu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2rpu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2rpu-trying-by-copy2buffertoimage4rpu: " + str(imagedestinationinfo2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2rpu = get2imageinfo4rpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2rpu-got-by-copy2buffertoimage4rpu: " + str(imagedestinationinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4rpu', 
            #  'METHOD': 'create2context4rpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4rpu at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4rpu at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2rpu-failed-by-copy2buffertoimage4rpu: " + str(imagedestinationinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2rpu ['IMAGEDESTINATION'] = imagedestinationinfo2rpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2rpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2rpu-trying-by-copy2buffertoimage4rpu: " + str(buffersourceinfo2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2rpu = get2bufferinfo4rpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2rpu-got-by-copy2buffertoimage4rpu: " + str(buffersourceinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4rpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4rpu', 
            #  'METHOD': 'create2buffer4rpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4rpu at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2rpu-failed-by-copy2buffertoimage4rpu: " + str(buffersourceinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2rpu ['BUFFERSOURCE'] = buffersourceinfo2rpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY RPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4rpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4rpu-trying-by-copy2buffertoimage4rpu: " + str(queue2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4rpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4rpu-got-by-copy2buffertoimage4rpu: " + str(queue2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4rpu-failed-by-copy2buffertoimage4rpu: " + str(queue2copybuffertoimage4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2rpu ['QUEUE'] = queue2copybuffertoimage4rpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY RPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2rpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2rpu-trying-by-copy2buffertoimage4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2rpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2rpu-got-by-copy2buffertoimage4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2rpu-failed-by-copy2buffertoimage4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2rpu ['PIPE'] = pipe2rpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2rpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2rpu-trying-by-copy2buffertoimage4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2rpu-got-by-copy2buffertoimage4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2rpu-failed-by-copy2buffertoimage4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2rpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2rpu-trying-by-copy2buffertoimage4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2rpu = get2eventinfo4rpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2rpu-got-by-copy2buffertoimage4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2rpu-failed-by-copy2buffertoimage4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2rpu ['EVENT'] = eventinfo2rpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2rpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2rpu-trying-by-copy2buffertoimage4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2rpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2rpu-set-by-copy2buffertoimage4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2rpu-failed-by-copy2buffertoimage4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2rpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-copy2buffertoimage4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2rpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2rpu-set-by-copy2buffertoimage4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-copy2buffertoimage4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2rpu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2rpu-trying-by-copy2buffertoimage4rpu: " + str(origin4destination2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2rpu['X'] = 100 
            origin4destination2rpu['Y'] = 100  
            origin4destination2rpu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2rpu-set-by-copy2buffertoimage4rpu to: " + str(origin4destination2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2rpu-failed-by-copy2buffertoimage4rpu: " + str(origin4destination2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2rpu ['DESTINATIONORIGIN'] = origin4destination2rpu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2rpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2rpu-trying-by-copy2buffertoimage4rpu: " + str(region4copy2rpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2rpu['X'] = 10 
            region4copy2rpu['Y'] = 10  
            region4copy2rpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2rpu-set-by-copy2buffertoimage4rpu to: " + str(region4copy2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2rpu-failed-by-copy2buffertoimage4rpu: " + str(region4copy2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2rpu ['REGION'] = region4copy2rpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2rpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2rpu-set-by-copy2buffertoimage4rpu to: " + str(offset4copy2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2rpu ['OFFSET'] = offset4copy2rpu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2rpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4rpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4rpu = trace2image4rpu                                                             # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4rpu-created-by-copy2buffertoimage4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4rpu"                                                        # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4rpu"                                                        # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2rpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2rpu-created-by-copy2buffertoimage4rpu to: " + str(task4copybuffertoimage2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2rpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2rpu-trying-by-copy2buffertoimage4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2rpu-got-by-copy2buffertoimage4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2rpu-failed-by-copy2buffertoimage4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2rpu ['CONTEXT'] = context2rpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2rpu-trying-by-copy2buffertoimage4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2rpu-got-by-copy2buffertoimage4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2rpu-failed-by-copy2buffertoimage4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2rpu ['DEVICE'] = device2rpu
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2rpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2rpu-trying-by-copy2buffertoimage4rpu: " + str(destinationimageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2rpu-got-by-copy2buffertoimage4rpu: " + str(destinationimageinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2rpu-failed-by-copy2buffertoimage4rpu: " + str(destinationimageinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2rpu ['IMAGEDESTINATION'] = destinationimageinfo2rpu         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2rpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2rpu-trying-by-copy2buffertoimage4rpu: " + str(sourcebufferinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2rpu = get2bufferinfo4rpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2rpu-got-by-copy2buffertoimage4rpu: " + str(sourcebufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2rpu-failed-by-copy2buffertoimage4rpu: " + str(sourcebufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2rpu ['BUFFERSOURCE'] = sourcebufferinfo2rpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY RPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4rpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4rpu-trying-by-copy2buffertoimage4rpu: " + str(queue2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4rpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4rpu-got-by-copy2buffertoimage4rpu: " + str(queue2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4rpu-failed-by-copy2buffertoimage4rpu: " + str(queue2copybuffertoimage4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2rpu ['QUEUE'] = queue2copybuffertoimage4rpu
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY RPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2rpu-trying-by-copy2buffertoimage4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2rpu-got-by-copy2buffertoimage4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2rpu-failed-by-copy2buffertoimage4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2rpu ['PIPE'] = pipe2rpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2rpu-trying-by-copy2buffertoimage4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2rpu-got-by-copy2buffertoimage4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2rpu-failed-by-copy2buffertoimage4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2rpu-trying-by-copy2buffertoimage4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2rpu-got-by-copy2buffertoimage4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2rpu-failed-by-copy2buffertoimage4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2rpu ['EVENT'] = eventinfo2rpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2rpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2rpu-trying-by-copy2buffertoimage4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2rpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2rpu-set-by-copy2buffertoimage4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2rpu-failed-by-copy2buffertoimage4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2rpu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-copy2buffertoimage4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2rpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2rpu-set-by-copy2buffertoimage4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-copy2buffertoimage4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2rpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2rpu-trying-by-copy2buffertoimage4rpu: " + str(destination4origin2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2rpu['X'] = 0 
                destination4origin2rpu['Y'] = 0  
                destination4origin2rpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2rpu-set-by-copy2buffertoimage4rpu to: " + str(destination4origin2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2rpu-failed-by-copy2buffertoimage4rpu: " + str(destination4origin2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2rpu ['DESTINATIONORIGIN'] = destination4origin2rpu
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2rpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2rpu-trying-by-copy2buffertoimage4rpu: " + str(region2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2rpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2rpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2rpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2rpu-set-by-copy2buffertoimage4rpu to: " + str(region2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2rpu-failed-by-copy2buffertoimage4rpu: " + str(region2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2rpu ['REGION'] = region2rpu
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2rpu = 0
            message2print = "{(Step 37A/70) set" + "; offset2rpu-set-by-copy2buffertoimage4rpu to: " + str(offset2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2rpu ['OFFSET'] = offset2rpu        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2rpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4rpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4rpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2rpu = False
    channels2rpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2rpu-trying-by-copy2buffertoimage4rpu: " + str(channels2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4rpu, publisherchannel2copybuffertoimage4rpu = task4copybuffertoimage2rpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4rpu-got-by-copy2buffertoimage4rpu: " + str(consumerchannel2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4rpu-got-by-copy2buffertoimage4rpu: " + str(publisherchannel2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2rpu = True
        
        control2copybuffertoimage4rpu['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4rpu
        
        
        control2copybuffertoimage4rpu['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4rpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2rpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4rpu = False
    queue2copybuffertoimage4rpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4rpu-trying-by-copy2buffertoimage4rpu: " + str(queue2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4rpu = task4copybuffertoimage2rpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4rpu-got-by-copy2buffertoimage4rpu: " + str(queue2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4rpu = True
        control2copybuffertoimage4rpu['QUEUE']  = queue2copybuffertoimage4rpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4rpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4rpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4rpu-trying-by-copy2buffertoimage4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4rpu = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4rpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4rpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4rpu, publisherchannel2copybuffertoimage4rpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4rpu-created-by-copy2buffertoimage4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4rpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4rpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4rpu-returning-by-copy2buffertoimage4rpu: " + str(control2copybuffertoimage4rpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4rpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4rpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4RPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4RPU ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4rpu(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4RPU : ', response4read2image)
#       queue2read4rpu      = response4read2image['QUEUE']
#       channel2readpublish4rpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4rpu = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4RPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4RPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())     
#   try:                               
#       message2readsend4rpu = "<copy4xi>copy2read4rpu</copy4xi>"
#       channel2readpublish4rpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4rpu.send(message2readsend4rpu)
#       print("(Step 17B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2readpublish4rpu, " MESSAGE2SEND4RPU: " , message2readsend4rpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4RPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2readimage4rpu(queue2read4rpu, channel2readconsume4rpu, channel2readpublish4rpu)
#       print("(Step 18D/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", manoeuvre2readimage4rpu)
#   except:
#       print("(Step 18Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())     
#   try:
#       channel2readconsume4rpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4rpu = channel2readconsume4rpu.recv()
#       print("(Step 19B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2readconsume4rpu, " CONTROL & COMMAND: ", message2readreceive4rpu)                                                                                                  
#       # <roger4xi>roger2read4rpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4RPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4RPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())     
#   try:                               
#       message2readsend4rpu = "<ready4xi>ready2read4rpu</ready4xi>"
#       channel2readpublish4rpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4rpu.send(message2readsend4rpu)
#       print("(Step 20B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2readpublish4rpu, " MESSAGE2SEND4RPU: " , message2readsend4rpu)                                                                                                
#       # <ready4xi>ready2read4rpu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4RPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2readimage4rpu(queue2read4rpu, channel2readconsume4rpu, channel2readpublish4rpu)
#       print("(Step 21B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", manoeuvre2readimage4rpu)
#   except:
#       print("(Step 21Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())     
#   try:
#       channel2readconsume4rpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4rpu = channel2readconsume4rpu.recv()
#       print("(Step 22B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2readconsume4rpu, " CONTROL & COMMAND: ", message2readreceive4rpu)                                                                                                  
#       # <ready4xi>ready2read4rpu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4RPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4RPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2readimage4rpu(queue2read4rpu, channel2readconsume4rpu, channel2readpublish4rpu)
#       print("(Step 25B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", manoeuvre2readimage4rpu)
#   except:
#       print("(Step 25Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4rpu = channel2readconsume4rpu.recv()
#       print("(Step 26D/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2readconsume4rpu, " CONTROL & COMMAND: ", message2readreceive4rpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4rpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4RPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2readimage4rpu(queue2read4rpu, channel2readconsume4rpu, channel2readpublish4rpu)
#       print("(Step 27B/700) OF RPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4RPU ", response2manoeuvre4rpu)
#   except:
#       print("(Step 27Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4rpu(task4imageread2rpu=None):

    '''
    reads 2D images for RPU
    '''
    global rpu2image4xi                                                     # 2D-IMAGES FOR RPU  
    global control2readimage4rpu                                              # COMMAND AND CONTROL FOR RPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4rpu"
    method2print = "read2image4rpu"

    message2print = "{(Step 1A/33) started" + "; read2image4rpu-started: " + str(read2image4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4rpu at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2rpu-to-read2image4rpu: " + str(task4imageread2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE RPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if rpu2image4xi is None:
            rpu2image4xi = create2image4rpu()                                                                              # INITIALIZE IMAGE FOR RPU 
            message2print = "{(Step 1C/70) created" + "; rpu2image4xi-created-by-read2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4rpu-skipped-by-read2image4rpu: " + str(read2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            rpu2image4xi = create2image4rpu()
            message2print = "{(Step 1E/70) set" + "; rpu2image4xi-set-by-read2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            rpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; rpu2image4xi-yielded-by-read2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2rpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2rpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2rpu-created-by-read2image4rpu to: " + str(task4imageread2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4rpu = trace2image4rpu                                                              # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4rpu-created-by-read2image4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4rpu at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4rpu"                                                          # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4rpu"                                                          # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4rpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4rpu', 
            #  'METHOD': 'read2image4rpu'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2rpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2rpu-created-by-read2image4rpu to: " + str(task4imageread2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4rpu at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4rpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4rpu', 
            #  'METHOD': 'read2image4rpu'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2rpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2rpu-trying-by-read2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2rpu-got-by-read2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2rpu-failed-by-read2image4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2rpu ['CONTEXT'] = context2rpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2rpu-trying-by-read2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2rpu-got-by-read2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2rpu-failed-by-read2image4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2rpu ['DEVICE'] = device2rpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2rpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2rpu-trying-by-read2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2rpu-got-by-read2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2rpu-failed-by-read2image4rpu: " + str(imageinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2rpu ['IMAGE'] = imageinfo2rpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY RPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2rpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2rpu-trying-by-read2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2rpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2rpu-got-by-read2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2rpu-failed-by-read2image4rpu: " + str(queue2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2rpu ['QUEUE'] = queue2rpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY RPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2rpu-trying-by-read2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2rpu-got-by-read2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2rpu-failed-by-read2image4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2rpu ['PIPE'] = pipe2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2rpu-trying-by-read2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2rpu-got-by-read2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2rpu-failed-by-read2image4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY RPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2rpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2rpu-trying-by-read2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2rpu = get2bufferinfo4rpu()                                                                   # SYSTEM CREATES BUFFER FOR RPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2rpu-got-by-read2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2rpu-failed-by-read2image4rpu: " + str(bufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2rpu ['BUFFER'] = bufferinfo2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2rpu-trying-by-read2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2rpu-got-by-read2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2rpu-failed-by-read2image4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2rpu ['EVENT'] = eventinfo2rpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2rpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2rpu-trying-by-read2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2rpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2rpu-set-by-read2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2rpu-failed-by-read2image4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2rpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-read2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2rpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2rpu-set-by-read2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-read2image4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2rpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2rpu-trying-by-read2image4rpu: " + str(blocking2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2rpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2rpu-set-by-read2image4rpu to: " + str(blocking2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2rpu-failed-by-read2image4rpu: " + str(blocking2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2rpu ['BLOCKING'] = blocking2rpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2rpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2rpu-trying-by-read2image4rpu: " + str(origin2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2rpu['X'] = 0 
                origin2rpu['Y'] = 0  
                origin2rpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2rpu-set-by-read2image4rpu to: " + str(origin2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2rpu-failed-by-read2image4rpu: " + str(origin2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2rpu ['ORIGIN'] = origin2rpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2rpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2rpu-trying-by-read2image4rpu: " + str(region2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2rpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2rpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2rpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2rpu-set-by-read2image4rpu to: " + str(region2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2rpu-failed-by-read2image4rpu: " + str(region2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2rpu ['REGION'] = region2rpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2rpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2rpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2rpu-trying-by-read2image4rpu: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2rpu = int(width2rpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2rpu-set-by-read2image4rpu to: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2rpu-failed-by-read2image4rpu: " + str(rowpitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2rpu ['ROWPITCH'] = rowpitch2rpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2rpu = None
            height2rpu = 800
            size2rpu = int(width2rpu * height2rpu)                                                                  # CALCULATED MEMORY
            if size2rpu == 0:
                size2rpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2rpu-trying-by-read2image4rpu: " + str(pointer2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2rpu = Arena4Xi(size2rpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2rpu-got-by-read2image4rpu: " + str(arena2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2rpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4rpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2rpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4rpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2rpu:
                    size2rpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2rpu-set-by-read2buffer4rpu to: " + str(size2rpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2rpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2rpu-set-by-read2buffer4rpu to: " + str(pointer2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2rpu-failed-by-read2image4rpu: " + str(pointer2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2rpu ['POINTER'] = pointer2rpu                            
            task4imageread2rpu ['SIZE']    = size2rpu                                                               # AUTOMATIC BY RPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2rpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2rpu-trying-by-read2image4rpu: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2rpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2rpu-set-by-read2image4rpu to: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2rpu-failed-by-read2image4rpu: " + str(slicepitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2rpu ['SLICEPITCH'] = slicepitch2rpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2rpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4rpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4rpu = trace2image4rpu                                                              # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4rpu-created-by-read2image4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4rpu"                                                          # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4rpu"                                                          # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2rpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2rpu-created-by-read2image4rpu to: " + str(task4imageread2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2rpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2rpu-trying-by-read2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2rpu-got-by-read2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2rpu-failed-by-read2image4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2rpu ['CONTEXT'] = context2rpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2rpu-trying-by-read2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2rpu-got-by-read2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2rpu-failed-by-read2image4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2rpu ['DEVICE'] = device2rpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2rpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2rpu-trying-by-read2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2rpu-got-by-read2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2rpu-failed-by-read2image4rpu: " + str(imageinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2rpu ['IMAGE'] = imageinfo2rpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY RPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2rpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2rpu-trying-by-read2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2rpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2rpu-got-by-read2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2rpu-failed-by-read2image4rpu: " + str(queue2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2rpu ['QUEUE'] = queue2rpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY RPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2rpu-trying-by-read2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2rpu-got-by-read2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2rpu-failed-by-read2image4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2rpu ['PIPE'] = pipe2rpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2rpu-trying-by-read2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2rpu-got-by-read2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2rpu-failed-by-read2image4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY RPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2rpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2rpu-trying-by-read2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2rpu = get2bufferinfo4rpu()                                                                   # SYSTEM CREATES BUFFER FOR RPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2rpu-got-by-read2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2rpu-failed-by-read2image4rpu: " + str(bufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2rpu ['BUFFER'] = bufferinfo2rpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2rpu-trying-by-read2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2rpu-got-by-read2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2rpu-failed-by-read2image4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2rpu ['EVENT'] = eventinfo2rpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2rpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2rpu-trying-by-read2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2rpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2rpu-set-by-read2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2rpu-failed-by-read2image4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2rpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-read2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2rpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2rpu-set-by-read2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-read2image4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2rpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2rpu-trying-by-read2image4rpu: " + str(blocking2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2rpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2rpu-set-by-read2image4rpu to: " + str(blocking2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2rpu-failed-by-read2image4rpu: " + str(blocking2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2rpu ['BLOCKING'] = blocking2rpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2rpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2rpu-trying-by-read2image4rpu: " + str(origin2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2rpu['X'] = 0 
                origin2rpu['Y'] = 0  
                origin2rpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2rpu-set-by-read2image4rpu to: " + str(origin2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2rpu-failed-by-read2image4rpu: " + str(origin2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2rpu ['ORIGIN'] = origin2rpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2rpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2rpu-trying-by-read2image4rpu: " + str(region2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2rpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2rpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2rpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2rpu-set-by-read2image4rpu to: " + str(region2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2rpu-failed-by-read2image4rpu: " + str(region2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2rpu ['REGION'] = region2rpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2rpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2rpu-trying-by-read2image4rpu: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2rpu = int(width2rpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2rpu-set-by-read2image4rpu to: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2rpu-failed-by-read2image4rpu: " + str(rowpitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2rpu ['ROWPITCH'] = rowpitch2rpu     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2rpu = None
            size2rpu = int(width2rpu * height2rpu)                                                                  # CALCULATED MEMORY
            if size2rpu == 0:
                size2rpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2rpu-trying-by-read2image4rpu: " + str(pointer2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2rpu = Arena4Xi(size2rpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2rpu-got-by-read2image4rpu: " + str(arena2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2rpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4rpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2rpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4rpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2rpu:
                    size2rpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2rpu-set-by-read2buffer4rpu to: " + str(size2rpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2rpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2rpu-set-by-read2buffer4rpu to: " + str(pointer2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2rpu-failed-by-read2image4rpu: " + str(pointer2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2rpu ['POINTER'] = pointer2rpu                            
            task4imageread2rpu ['SIZE']    = size2rpu                                                               # AUTOMATIC BY RPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2rpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2rpu-trying-by-read2image4rpu: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2rpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2rpu-set-by-read2image4rpu to: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2rpu-failed-by-read2image4rpu: " + str(slicepitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2rpu ['SLICEPITCH'] = slicepitch2rpu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2rpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4rpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2rpu-yielded-by-read2image4rpu: " + str(task4imageread2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4rpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2rpu = False
    channels2rpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2rpu-trying-by-read2image4rpu: " + str(channels2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2rpu, publisherchannel2rpu = task4imageread2rpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2rpu-got-by-read2image4rpu: " + str(consumerchannel2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2rpu-got-by-read2image4rpu: " + str(publisherchannel2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2rpu = True
        
        control2readimage4rpu['CONSUMERCHANNEL']  = consumerchannel2rpu
        
        
        control2readimage4rpu['PUBLISHERCHANNEL'] = publisherchannel2rpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2rpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2rpu = False
    queue2rpu = None
    message2print = "{(Step 41A/70) trying" + "; queue2rpu-trying-by-read2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2rpu = task4imageread2rpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2rpu-got-by-read2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2rpu = True
        control2readimage4rpu['QUEUE']  = queue2rpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2rpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4rpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4rpu-trying-by-read2image4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4rpu = DummyProcess4Xi(target=manoeuvre2readimage4rpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2rpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2rpu, publisherchannel2rpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4rpu-created-by-read2image4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4rpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4rpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4rpu-returning-by-read2image4rpu: " + str(control2readimage4rpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4rpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4rpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4RPU ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4rpu(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4RPU : ', response4map2image)
#    
#       queue4map2image4rpu      = response4map2image['QUEUE']
#       channel4map2imagepublish4rpu = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4rpu = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4RPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4RPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4RPU ', time4xi())      
#   try:                               
#       message2send4rpu = "<copy4xi>copy2mapimage4rpu</copy4xi>"
#       channel4map2imagepublish4rpu.send(message2send4rpu)
#       print("(Step 93B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel4map2imagepublish4rpu, " MESSAGE2SEND4RPU: " , message2send4rpu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4RPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4RPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4rpu is True:
#       print("(Step 94A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4rpu(queue4map2image4rpu, channel4map2imageconsume4rpu, channel4map2imagepublish4rpu)
#           print("(Step 94B/700) OF RPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4RPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4RPU ', time4xi())      
#   try:
#       message2receive4rpu = channel4map2imageconsume4rpu.recv()
#       print("(Step 95B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel4map2imageconsume4rpu, " CONTROL & COMMAND: ", message2receive4rpu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4RPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4RPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4RPU ', time4xi())     
#   try:                               
#       message2send4rpu = "<ready4xi>ready2mapimage4rpu</ready4xi>"
#       channel4map2imagepublish4rpu.send(message2send4rpu)
#       print("(Step 96B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel4map2imagepublish4rpu, " MESSAGE2SEND4RPU: " , message2send4rpu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4RPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4RPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4rpu is True:
#       print("(Step 97A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4rpu(queue4map2image4rpu, channel4map2imageconsume4rpu, channel4map2imagepublish4rpu)
#           print("(Step 97B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", manoeuvre2copyimage4rpu)
#       except:
#           print("(Step 97Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4RPU ', time4xi())     
#   try:
#       message2receive4rpu = channel4map2imageconsume4rpu.recv()
#       print("(Step 98B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel4map2imageconsume4rpu, " CONTROL & COMMAND: ", message2receive4rpu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4RPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE RPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF RPU2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE RPUXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF RPU2IMAGE4XI5TEST - FILENO: ", file2rpu.fileno( ))  
#       print("(Step 99C/700) OF RPU2IMAGE4XI5TEST - MODE: ", file2rpu.mode)
#       print("(Step 99D/700) OF RPU2IMAGE4XI5TEST - NAME: ", file2rpu.name)
#       offset2rpu = 10
#       file2rpu.seek(offset2rpu)
#       print("(Step 99E/700) OF RPU2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2rpu)
#       file2rpu.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4RPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4rpu is True:
#       print("(Step 101A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4rpu(queue4map2image4rpu, channel4map2imageconsume4rpu, channel4map2imagepublish4rpu)
#           print("(Step 101B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", manoeuvre2copyimage4rpu)
#       except:
#           print("(Step 101Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4rpu = channel4map2imageconsume4rpu.recv()
#       print("(Step 102B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel4map2imageconsume4rpu, " CONTROL & COMMAND: ", message2receive4rpu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4RPU  - LET MANOEUVRE2COPYIMAGE4RPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4rpu is True:
#       print("(Step 103A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4rpu(queue4map2image4rpu, channel4map2imageconsume4rpu, channel4map2imagepublish4rpu)
#           print("(Step 103B/700) OF RPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4RPU ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4rpu(task4imagemap2rpu=None):

    '''
    maps 2D images for RPU
    '''
    global rpu2image4xi                                                                                             # 2D-IMAGES FOR RPU  
    global control2mapimage4rpu                                                                                     # COMMAND AND CONTROL FOR RPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4rpu"
    method2print = "map2image4rpu"

    message2print = "{(Step 1A/33) started" + "; map2image4rpu-started: " + str(map2image4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2rpu-to-map2image4rpu: " + str(task4imagemap2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE RPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if rpu2image4xi is None:
            rpu2image4xi = create2image4rpu()                                                                              # INITIALIZE IMAGE FOR RPU 
            message2print = "{(Step 1C/70) created" + "; rpu2image4xi-created-by-map2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4rpu-skipped-by-map2image4rpu: " + str(map2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            rpu2image4xi = create2image4rpu()
            message2print = "{(Step 1E/70) set" + "; rpu2image4xi-set-by-map2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            rpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; rpu2image4xi-yielded-by-map2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2rpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2rpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2rpu-created-by-map2image4rpu to: " + str(task4imagemap2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4rpu = trace2image4rpu                                                              # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4rpu-created-by-map2image4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4rpu"                                                          # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4rpu"                                                          # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2rpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2rpu-created-by-map2image4rpu to: " + str(task4imagemap2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2rpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2rpu-trying-by-map2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2rpu-got-by-map2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2rpu-failed-by-map2image4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2rpu ['CONTEXT'] = context2rpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2rpu-trying-by-map2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2rpu-got-by-map2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2rpu-failed-by-map2image4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2rpu ['DEVICE'] = device2rpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2rpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2rpu-trying-by-map2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2rpu-got-by-map2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2rpu-failed-by-map2image4rpu: " + str(imageinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2rpu ['IMAGE'] = imageinfo2rpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY RPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2rpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2rpu-trying-by-map2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2rpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2rpu-got-by-map2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2rpu-failed-by-map2image4rpu: " + str(queue2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2rpu ['QUEUE'] = queue2rpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY RPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2rpu-trying-by-map2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2rpu-got-by-map2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2rpu-failed-by-map2image4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2rpu ['PIPE'] = pipe2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2rpu-trying-by-map2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2rpu-got-by-map2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2rpu-failed-by-map2image4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY RPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2rpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2rpu-trying-by-map2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2rpu = get2bufferinfo4rpu()                                                                   # SYSTEM CREATES BUFFER FOR RPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2rpu-got-by-map2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2rpu-failed-by-map2image4rpu: " + str(bufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2rpu ['BUFFER'] = bufferinfo2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2rpu-trying-by-map2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2rpu-got-by-map2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2rpu-failed-by-map2image4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2rpu ['EVENT'] = eventinfo2rpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2rpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2rpu-trying-by-map2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2rpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2rpu-set-by-map2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2rpu-failed-by-map2image4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2rpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-map2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2rpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2rpu-set-by-map2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-map2image4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2rpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2rpu-trying-by-map2image4rpu: " + str(blocking2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2rpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2rpu-set-by-map2image4rpu to: " + str(blocking2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2rpu-failed-by-map2image4rpu: " + str(blocking2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2rpu ['BLOCKING'] = blocking2rpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2rpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2rpu-trying-by-map2image4rpu: " + str(origin2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2rpu['X'] = 0 
                origin2rpu['Y'] = 0  
                origin2rpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2rpu-set-by-map2image4rpu to: " + str(origin2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2rpu-failed-by-map2image4rpu: " + str(origin2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2rpu ['ORIGIN'] = origin2rpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2rpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2rpu-trying-by-map2image4rpu: " + str(region2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2rpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2rpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2rpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2rpu-set-by-map2image4rpu to: " + str(region2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2rpu-failed-by-map2image4rpu: " + str(region2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2rpu ['REGION'] = region2rpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2rpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2rpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2rpu-trying-by-map2image4rpu: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2rpu = int(width2rpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2rpu-set-by-map2image4rpu to: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2rpu-failed-by-map2image4rpu: " + str(rowpitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2rpu ['ROWPITCH'] = rowpitch2rpu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2rpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2rpu-trying-by-map2image4rpu: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2rpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2rpu-set-by-map2image4rpu to: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2rpu-failed-by-map2image4rpu: " + str(slicepitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2rpu ['SLICEPITCH'] = slicepitch2rpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2rpu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2rpu-trying-by-map2image4rpu: " + str(flags2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2rpu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2rpu-set-by-map2image4rpu to: " + str(flags2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2rpu-failed-by-map2image4rpu: " + str(flags2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2rpu ['FLAGS'] = flags2rpu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2rpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4rpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4rpu = trace2image4rpu                                                              # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4rpu-created-by-map2image4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4rpu"                                                          # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4rpu"                                                          # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2rpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2rpu-created-by-map2image4rpu to: " + str(task4imagemap2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2rpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2rpu-trying-by-map2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2rpu-got-by-map2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2rpu-failed-by-map2image4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2rpu ['CONTEXT'] = context2rpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2rpu-trying-by-map2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2rpu-got-by-map2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2rpu-failed-by-map2image4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2rpu ['DEVICE'] = device2rpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2rpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2rpu-trying-by-map2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2rpu-got-by-map2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2rpu-failed-by-map2image4rpu: " + str(imageinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2rpu ['IMAGE'] = imageinfo2rpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY RPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2rpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2rpu-trying-by-map2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2rpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2rpu-got-by-map2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2rpu-failed-by-map2image4rpu: " + str(queue2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2rpu ['QUEUE'] = queue2rpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY RPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2rpu-trying-by-map2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2rpu-got-by-map2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2rpu-failed-by-map2image4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2rpu ['PIPE'] = pipe2rpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2rpu-trying-by-map2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2rpu-got-by-map2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2rpu-failed-by-map2image4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY RPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2rpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2rpu-trying-by-map2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2rpu = get2bufferinfo4rpu()                                                                   # SYSTEM CREATES BUFFER FOR RPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2rpu-got-by-map2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2rpu-failed-by-map2image4rpu: " + str(bufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2rpu ['BUFFER'] = bufferinfo2rpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2rpu-trying-by-map2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2rpu-got-by-map2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2rpu-failed-by-map2image4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2rpu ['EVENT'] = eventinfo2rpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2rpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2rpu-trying-by-map2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2rpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2rpu-set-by-map2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2rpu-failed-by-map2image4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2rpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-map2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2rpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2rpu-set-by-map2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-map2image4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2rpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2rpu-trying-by-map2image4rpu: " + str(blocking2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2rpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2rpu-set-by-map2image4rpu to: " + str(blocking2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2rpu-failed-by-map2image4rpu: " + str(blocking2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2rpu ['BLOCKING'] = blocking2rpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2rpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2rpu-trying-by-map2image4rpu: " + str(origin2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2rpu['X'] = 0 
                origin2rpu['Y'] = 0  
                origin2rpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2rpu-set-by-map2image4rpu to: " + str(origin2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2rpu-failed-by-map2image4rpu: " + str(origin2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2rpu ['ORIGIN'] = origin2rpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2rpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2rpu-trying-by-map2image4rpu: " + str(region2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2rpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2rpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2rpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2rpu-set-by-map2image4rpu to: " + str(region2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2rpu-failed-by-map2image4rpu: " + str(region2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2rpu ['REGION'] = region2rpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2rpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2rpu-trying-by-map2image4rpu: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2rpu = int(width2rpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2rpu-set-by-map2image4rpu to: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2rpu-failed-by-map2image4rpu: " + str(rowpitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2rpu ['ROWPITCH'] = rowpitch2rpu     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2rpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2rpu-trying-by-map2image4rpu: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2rpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2rpu-set-by-map2image4rpu to: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2rpu-failed-by-map2image4rpu: " + str(slicepitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2rpu ['SLICEPITCH'] = slicepitch2rpu
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2rpu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2rpu-trying-by-map2image4rpu: " + str(flags2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2rpu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2rpu-set-by-map2image4rpu to: " + str(flags2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2rpu-failed-by-map2image4rpu: " + str(flags2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2rpu ['FLAGS'] = flags2rpu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2rpu.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4rpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2rpu-yielded-by-map2image4rpu: " + str(task4imagemap2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4rpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2rpu = False
    channels2rpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2rpu-trying-by-map2image4rpu: " + str(channels2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2rpu, publisherchannel2rpu = task4imagemap2rpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2rpu-got-by-map2image4rpu: " + str(consumerchannel2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2rpu-got-by-map2image4rpu: " + str(publisherchannel2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2rpu = True
        
        control2mapimage4rpu['CONSUMERCHANNEL']  = consumerchannel2rpu
        
        
        control2mapimage4rpu['PUBLISHERCHANNEL'] = publisherchannel2rpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2rpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2rpu = False
    queue2rpu = None
    message2print = "{(Step 41A/70) trying" + "queue2rpu-trying-by-map2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2rpu = task4imagemap2rpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2rpu-got-by-map2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2rpu = True
        control2mapimage4rpu['QUEUE']  = queue2rpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2rpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4rpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4rpu-trying-by-map2image4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4rpu = DummyProcess4Xi(target=manoeuvre2mapimage4rpu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2rpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2rpu, publisherchannel2rpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4rpu-created-by-map2image4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4rpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4rpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4rpu-returning-by-map2image4rpu: " + str(control2mapimage4rpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4rpu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4rpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4RPU - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4rpu is True:
#       print("(Step 42A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4rpu(queue2copy4rpu, channel2copyconsume4rpu, channel2copypublish4rpu)
#           print("(Step 42B/700) OF RPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4RPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4RPU': '<roger4xi>roger2copy4rpu</roger4xi>', 'ACK2INFO4RPU': False, 'MESSAGE2GET4RPU': '<default4xi>default2copy4rpu</default4xi>', 
#           #  'MESSAGE2RECEIVE4RPU': '<copy4xi>copy2copy4rpu</copy4xi>', 
#           'CODE2ACK4RPU': ' 0D-1C-', 'MESSAGE2PUT4RPU': '<default4xi>default2copy4rpu</default4xi>', 'CODE2NACK4RPU': ' ', 'STATE2MANOEUVRE4RPU': 1}
#        except:
#           print("(Step 42Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4rpu(queue2copymanoeuvre4rpu, consumerchannel2copymanoeuvre4rpu, publisherchannel2copymanoeuvre4rpu):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4rpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4rpu                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4rpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4rpu                                 
    global message2copyimageput4rpu
    global message2copyimagereceive4rpu
    global message2copyimagesend4rpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4rpu" 
    method2print = "manoeuvre2copyimage4rpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4rpu-started: " + str(manoeuvre2copyimage4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4rpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4rpu-to-manoeuvre2copyimage4rpu: " + str(queue2copymanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4rpu-to-manoeuvre2copyimage4rpu: " + str(consumerchannel2copymanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4rpu-to-manoeuvre2copyimage4rpu: " + str(publisherchannel2copymanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4rpu-try-by-manoeuvre2copyimage4rpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4rpu is None:
            state2copyimage4rpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4rpu-initialized-by-manoeuvre2copyimage4rpu to: " + str(state2copyimage4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4rpu-used-by-manoeuvre2copyimage4rpu: " + str(state2copyimage4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4rpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4rpu-initialized-by-manoeuvre2copyimage4rpu to: " + str(state2copyimage4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4rpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4rpu-initialized-by-manoeuvre2copyimage4rpu to: " + str(ack2copyimageinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4rpu     = '<default4xi>default2copy4rpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4rpu-initialized-by-manoeuvre2copyimage4rpu to: " + str(message2copyimageget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4rpu</default4xi>
        message2copyimageput4rpu     = '<default4xi>default2copy4rpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4rpu-initialized-by-manoeuvre2copyimage4rpu to: " + str(message2copyimageput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4rpu</default4xi>
        message2copyimagereceive4rpu = '<default4xi>default2copy4rpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4rpu-initialized-by-manoeuvre2copyimage4rpu to: " + str(message2copyimagereceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4rpu</default4xi>
        message2copyimagesend4rpu    = '<default4xi>default2copy4rpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4rpu-initialized-by-manoeuvre2copyimage4rpu to: " + str(message2copyimagesend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4rpu</default4xi>
        max2copyimagestate4rpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4rpu-initialized-by-manoeuvre2copyimage4rpu to: " + str( max2copyimagestate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4rpu = state2copyimage4rpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4rpu-added-by-manoeuvre2copyimage4rpu: " + str(state2copyimage4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4rpu > max2copyimagestate4rpu:
        state2copyimage4rpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4rpu-set-by-manoeuvre2copyimage4rpu to: " + str(state2copyimage4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4rpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4rpu-set-by-manoeuvre2copyimage4rpu to: " + str(ack2copyimageinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4rpu     = '<default4xi>default2copy4rpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4rpu-set-by-manoeuvre2copyimage4rpu to: " + str(message2copyimageget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4rpu</default4xi>
        message2copyimageput4rpu     = '<default4xi>default2copy4rpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4rpu-set-by-manoeuvre2copyimage4rpu to: " + str(message2copyimageput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4rpu</default4xi>
        message2copyimagereceive4rpu = '<default4xi>default2copy4rpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4rpu-set-by-manoeuvre2copyimage4rpu to: " + str(message2copyimagereceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4rpu</default4xi>
        message2copyimagesend4rpu    = '<default4xi>default2copy4rpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4rpu-set-by-manoeuvre2copyimage4rpu to: " + str(message2copyimagesend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4rpu</default4xi>
        max2copyimagestate4rpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4rpu-set-by-manoeuvre2copyimage4rpu to: " + str( max2copyimagestate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4rpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4rpu = consumerchannel2copymanoeuvre4rpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4rpu-received-by-manoeuvre2copyimage4rpu: " + str(message2copyimagereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4rpu</copy4xi>
                         
            if message2copyimagereceive4rpu == '<copy4xi>copy2copy4rpu</copy4xi>':
                message2copyimagesend4rpu = '<roger4xi>roger2copy4rpu</roger4xi>'
            else:
                message2copyimagesend4rpu = '<negative4xi><negative2copy4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4rpu.send(message2copyimagesend4rpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4rpu-sent-by-manoeuvre2copyimage4rpu: " + str(message2copyimagesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4rpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4rpu-failed-by-manoeuvre2copyimage4rpu: " + str(message2copyimagereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4rpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4rpu = consumerchannel2copymanoeuvre4rpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4rpu-received-by-manoeuvre2copyimage4rpu: " + str(message2copyimagereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4rpu</ready4xi>
                         
            if message2copyimagereceive4rpu == '<ready4xi>ready2copy4rpu</ready4xi>':
                message2copyimagesend4rpu = '<ready4xi>ready2copy4rpu</ready4xi>'
            else:
                message2copyimagesend4rpu = '<negative4xi><negative2copy4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4rpu.send(message2copyimagesend4rpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4rpu-sent-by-manoeuvre2copyimage4rpu: " + str(message2copyimagesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4rpu-failed-by-manoeuvre2copyimage4rpu: " + str(message2copyimagereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4rpu == 3:    
        info2image4rpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4rpu-try-by-manoeuvre2copyimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4rpu = queue2copymanoeuvre4rpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4rpu-got-by-manoeuvre2copyimage4rpu: " + str(info2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4rpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4rpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4rpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4rpu-failed-by-manoeuvre2copyimage4rpu: " + str(queue2copymanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4rpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4rpu is True:
                message2copyimagesend4rpu = '<ack4xi>ack2imageinfo4rpu</ack4xi>'
            else:
                message2copyimagesend4rpu = '<nack4xi>nack2imageinfo4rpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4rpu-set-by-manoeuvre2copyimage4rpu to: " + str(message2copyimagesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4rpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4rpu-failed-by-manoeuvre2copyimage4rpu: " + str(message2copyimagesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4rpu-calling-by-manoeuvre2copyimage4rpu: " + str(publisherchannel2copymanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4rpu-passing-to-send-by-manoeuvre2copyimage4rpu: " + str(message2copyimagesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4rpu-failed-by-manoeuvre2copyimage4rpu: " + str(message2copyimagesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4rpu.send(message2copyimagesend4rpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4rpu-sent-by-manoeuvre2copyimage4rpu: " + str(message2copyimagesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4rpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4rpu-failed-by-manoeuvre2copyimage4rpu: " + str(message2copyimagesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4RPU']        = ack2copyimageinfo4rpu
    response2manouvre4copy['CODE2ACK4RPU']        = code2ack
    response2manouvre4copy['CODE2NACK4RPU']       = code2nack
    response2manouvre4copy['MESSAGE2GET4RPU']     = message2copyimageget4rpu
    response2manouvre4copy['MESSAGE2PUT4RPU']     = message2copyimageput4rpu
    response2manouvre4copy['MESSAGE2RECEIVE4RPU'] = message2copyimagereceive4rpu
    response2manouvre4copy['MESSAGE2SEND4RPU']    = message2copyimagesend4rpu
    response2manouvre4copy['STATE2MANOEUVRE4RPU'] = state2copyimage4rpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4rpu: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4RPU': '<roger4xi>roger2copy4rpu</roger4xi>',
    #   'ACK2INFO4RPU': False,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<copy4xi>copy2copy4rpu</copy4xi>',
    #   'CODE2ACK4RPU': ' 0D-1C-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4RPU': '<ready4xi>ready2copy4rpu</ready4xi>',
    #   'ACK2INFO4RPU': False,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2copy4rpu</ready4xi>',
    #   'CODE2ACK4RPU': ' 0D-2C-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4RPU': '<ack4xi>ack2imageinfo4rpu</ack4xi>',
    #   'ACK2INFO4RPU': True,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2copy4rpu</ready4xi>',
    #   'CODE2ACK4RPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ', 'STATE2MANOEUVRE4RPU': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'ACK2INFO4RPU': False,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'CODE2ACK4RPU': ' 0D-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copy4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ', 'STATE2MANOEUVRE4RPU': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4rpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4RPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4rpu is True:
#       print("(Step 62A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4rpu(queue4copy2imagetobuffer4rpu, channel4copy2imagetobufferconsume4rpu, channel4copy2imagetobufferpublish4rpu)
#           print("(Step 62B/700) OF RPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4RPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4RPU': '<roger4xi>roger2copyimagetobuffer4rpu</roger4xi>',
#           #   'ACK2INFO4RPU': False,
#           #   'MESSAGE2GET4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
#           #   'MESSAGE2RECEIVE4RPU': '<copy4xi>copy2copyimagetobuffer4rpu</copy4xi>',
#           #   'CODE2ACK4RPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
#           #   'CODE2NACK4RPU': ' ',
#           #   'STATE2MANOEUVRE4RPU': 1}
#        except:
#           print("(Step 62Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4rpu(queue2copyimagetobuffermanoeuvre4rpu, consumerchannel2copyimagetobuffermanoeuvre4rpu, publisherchannel2copyimagetobuffermanoeuvre4rpu):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4rpu
    '''
    global state2copyimagetobuffer4rpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4rpu                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4rpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4rpu                                 
    global message2copyimagetobufferput4rpu
    global message2copyimagetobufferreceive4rpu
    global message2copyimagetobuffersend4rpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4rpu" 
    method2print = "manoeuvre2copyimagetobuffer4rpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4rpu-started: " + str(manoeuvre2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4rpu-to-manoeuvre2copyimagetobuffer4rpu: " + str(queue2copyimagetobuffermanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4rpu-to-manoeuvre2copyimagetobuffer4rpu: " + str(consumerchannel2copyimagetobuffermanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4rpu-to-manoeuvre2copyimagetobuffer4rpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4rpu-try-by-manoeuvre2copyimagetobuffer4rpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4rpu is None:
            state2copyimagetobuffer4rpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4rpu-initialized-by-manoeuvre2copyimagetobuffer4rpu to: " + str(state2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4rpu-used-by-manoeuvre2copyimagetobuffer4rpu: " + str(state2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4rpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4rpu-initialized-by-manoeuvre2copyimagetobuffer4rpu to: " + str(state2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4rpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4rpu-initialized-by-manoeuvre2copyimagetobuffer4rpu to: " + str(ack2copyimagetobufferinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4rpu     = '<default4xi>default2copyimagetobuffer4rpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4rpu-initialized-by-manoeuvre2copyimagetobuffer4rpu to: " + str(message2copyimagetobufferget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4rpu     = '<default4xi>default2copyimagetobuffer4rpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4rpu-initialized-by-manoeuvre2copyimagetobuffer4rpu to: " + str(message2copyimagetobufferput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4rpu = '<default4xi>default2copyimagetobuffer4rpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4rpu-initialized-by-manoeuvre2copyimagetobuffer4rpu to: " + str(message2copyimagetobufferreceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4rpu    = '<default4xi>default2copyimagetobuffer4rpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4rpu-initialized-by-manoeuvre2copyimagetobuffer4rpu to: " + str(message2copyimagetobuffersend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4rpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4rpu-initialized-by-manoeuvre2copyimagetobuffer4rpu to: " + str( max2copyimagetobufferstate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4rpu = state2copyimagetobuffer4rpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4rpu-added-by-manoeuvre2copyimagetobuffer4rpu: " + str(state2copyimagetobuffer4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4rpu > max2copyimagetobufferstate4rpu:
        state2copyimagetobuffer4rpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4rpu-set-by-manoeuvre2copyimagetobuffer4rpu to: " + str(state2copyimagetobuffer4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4rpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4rpu-set-by-manoeuvre2copyimagetobuffer4rpu to: " + str(ack2copyimagetobufferinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4rpu     = '<default4xi>default2copyimagetobuffer4rpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4rpu-set-by-manoeuvre2copyimagetobuffer4rpu to: " + str(message2copyimagetobufferget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4rpu     = '<default4xi>default2copyimagetobuffer4rpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4rpu-set-by-manoeuvre2copyimagetobuffer4rpu to: " + str(message2copyimagetobufferput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4rpu = '<default4xi>default2copyimagetobuffer4rpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4rpu-set-by-manoeuvre2copyimagetobuffer4rpu to: " + str(message2copyimagetobufferreceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4rpu    = '<default4xi>default2copyimagetobuffer4rpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4rpu-set-by-manoeuvre2copyimagetobuffer4rpu to: " + str(message2copyimagetobuffersend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4rpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4rpu-set-by-manoeuvre2copyimagetobuffer4rpu to: " + str( max2copyimagetobufferstate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4rpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4rpu = consumerchannel2copyimagetobuffermanoeuvre4rpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4rpu-received-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobufferreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4rpu == '<copy4xi>copy2copyimagetobuffer4rpu</copy4xi>': 
                message2copyimagetobuffersend4rpu = '<roger4xi>roger2copyimagetobuffer4rpu</roger4xi>'
            else:
                message2copyimagetobuffersend4rpu = '<negative4xi><negative2copyimagetobuffer4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4rpu.send(message2copyimagetobuffersend4rpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4rpu-sent-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobuffersend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4rpu-failed-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobufferreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4rpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4rpu = consumerchannel2copyimagetobuffermanoeuvre4rpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4rpu-received-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobufferreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4rpu == '<ready4xi>ready2copyimagetobuffer4rpu</ready4xi>':
                message2copyimagetobuffersend4rpu = '<ready4xi>ready2copyimagetobuffer4rpu</ready4xi>'
            else:
                message2copyimagetobuffersend4rpu = '<negative4xi><negative2copyimagetobuffer4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4rpu.send(message2copyimagetobuffersend4rpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4rpu-sent-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobuffersend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4rpu-failed-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobufferreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4rpu == 3:    
        info2image4rpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4rpu-try-by-manoeuvre2copyimagetobuffer4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4rpu = queue2copyimagetobuffermanoeuvre4rpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4rpu-got-by-manoeuvre2copyimagetobuffer4rpu: " + str(info2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4rpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4rpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4rpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4rpu-failed-by-manoeuvre2copyimagetobuffer4rpu: " + str(queue2copyimagetobuffermanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4rpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4rpu is True:
                message2copyimagetobuffersend4rpu = '<ack4xi>ack2imageinfo4rpu</ack4xi>'
            else:
                message2copyimagetobuffersend4rpu = '<nack4xi>nack2imageinfo4rpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4rpu-set-by-manoeuvre2copyimagetobuffer4rpu to: " + str(message2copyimagetobuffersend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4rpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4rpu-failed-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobuffersend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4rpu-calling-by-manoeuvre2copyimagetobuffer4rpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4rpu-passing-to-send-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobuffersend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4rpu-failed-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobuffersend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4rpu.send(message2copyimagetobuffersend4rpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4rpu-sent-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobuffersend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4rpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4rpu-failed-by-manoeuvre2copyimagetobuffer4rpu: " + str(message2copyimagetobuffersend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4RPU']        = ack2copyimagetobufferinfo4rpu
    response2manouvre4copyimagetobuffer['CODE2ACK4RPU']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4RPU']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4RPU']     = message2copyimagetobufferget4rpu
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4RPU']     = message2copyimagetobufferput4rpu
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4RPU'] = message2copyimagetobufferreceive4rpu
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4RPU']    = message2copyimagetobuffersend4rpu
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4RPU'] = state2copyimagetobuffer4rpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4rpu: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4RPU': '<roger4xi>roger2copyimagetobuffer4rpu</roger4xi>',
    #   'ACK2INFO4RPU': False,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<copy4xi>copy2copyimagetobuffer4rpu</copy4xi>',
    #   'CODE2ACK4RPU': ' 0D-1C-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4RPU': '<ready4xi>ready2copyimagetobuffer4rpu</ready4xi>',
    #   'ACK2INFO4RPU': False,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2copyimagetobuffer4rpu</ready4xi>',
    #   'CODE2ACK4RPU': ' 0D-2C-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4RPU': '<ack4xi>ack2imageinfo4rpu</ack4xi>',
    #   'ACK2INFO4RPU': True,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2copyimagetobuffer4rpu</ready4xi>',
    #   'CODE2ACK4RPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'ACK2INFO4RPU': False,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'CODE2ACK4RPU': ' 0D-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copyimagetobuffer4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4rpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4RPU - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4rpu is True:
#       print("(Step 82A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4rpu(queue4copybuffertoimage4rpu, channel4copybuffertoimageconsume4rpu, channel4copybuffertoimagepublish4rpu)
#           print("(Step 82B/700) OF RPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4RPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4RPU': '<roger4xi>roger2copybuffertoimage4rpu</roger4xi>',
#           #   'ACK2INFO4RPU': False,
#           #   'MESSAGE2GET4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
#           #   'MESSAGE2RECEIVE4RPU': '<copy4xi>copy2copybuffertoimage4rpu</copy4xi>',
#           #   'CODE2ACK4RPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
#           #   'CODE2NACK4RPU': ' ',
#           #   'STATE2MANOEUVRE4RPU': 1}
#       except:
#           print("(Step 82Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4rpu(queue2copybuffertoimagemanoeuvre4rpu, consumerchannel2copybuffertoimagemanoeuvre4rpu, publisherchannel2copybuffertoimagemanoeuvre4rpu):
    '''
    manoeuvres copy control and command for copy2buffertoimage4rpu
    '''
    global state2copybuffertoimage4rpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4rpu                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4rpu                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4rpu                                 
    global message2copybuffertoimageput4rpu
    global message2copybuffertoimagereceive4rpu
    global message2copybuffertoimagesend4rpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4rpu" 
    method2print = "manoeuvre2copybuffertoimage4rpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4rpu-started: " + str(manoeuvre2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4rpu-to-manoeuvre2copybuffertoimage4rpu: " + str(queue2copybuffertoimagemanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4rpu-to-manoeuvre2copybuffertoimage4rpu: " + str(consumerchannel2copybuffertoimagemanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4rpu-to-manoeuvre2copybuffertoimage4rpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4rpu-try-by-manoeuvre2copybuffertoimage4rpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4rpu is None:
            state2copybuffertoimage4rpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4rpu-initialized-by-manoeuvre2copybuffertoimage4rpu to: " + str(state2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4rpu-used-by-manoeuvre2copybuffertoimage4rpu: " + str(state2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4rpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4rpu-initialized-by-manoeuvre2copybuffertoimage4rpu to: " + str(state2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4rpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4rpu-initialized-by-manoeuvre2copybuffertoimage4rpu to: " + str(ack2copybuffertoimageinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4rpu     = '<default4xi>default2copybuffertoimage4rpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4rpu-initialized-by-manoeuvre2copybuffertoimage4rpu to: " + str(message2copybuffertoimageget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4rpu     = '<default4xi>default2copybuffertoimage4rpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4rpu-initialized-by-manoeuvre2copybuffertoimage4rpu to: " + str(message2copybuffertoimageput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4rpu = '<default4xi>default2copybuffertoimage4rpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4rpu-initialized-by-manoeuvre2copybuffertoimage4rpu to: " + str(message2copybuffertoimagereceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4rpu    = '<default4xi>default2copybuffertoimage4rpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4rpu-initialized-by-manoeuvre2copybuffertoimage4rpu to: " + str(message2copybuffertoimagesend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4rpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4rpu-initialized-by-manoeuvre2copybuffertoimage4rpu to: " + str( max2copybuffertoimagestate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4rpu = state2copybuffertoimage4rpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4rpu-added-by-manoeuvre2copybuffertoimage4rpu: " + str(state2copybuffertoimage4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4rpu > max2copybuffertoimagestate4rpu:
        state2copybuffertoimage4rpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4rpu-set-by-manoeuvre2copybuffertoimage4rpu to: " + str(state2copybuffertoimage4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4rpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4rpu-set-by-manoeuvre2copybuffertoimage4rpu to: " + str(ack2copybuffertoimageinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4rpu     = '<default4xi>default2copybuffertoimage4rpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4rpu-set-by-manoeuvre2copybuffertoimage4rpu to: " + str(message2copybuffertoimageget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4rpu     = '<default4xi>default2copybuffertoimage4rpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4rpu-set-by-manoeuvre2copybuffertoimage4rpu to: " + str(message2copybuffertoimageput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4rpu = '<default4xi>default2copybuffertoimage4rpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4rpu-set-by-manoeuvre2copybuffertoimage4rpu to: " + str(message2copybuffertoimagereceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4rpu    = '<default4xi>default2copybuffertoimage4rpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4rpu-set-by-manoeuvre2copybuffertoimage4rpu to: " + str(message2copybuffertoimagesend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4rpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4rpu-set-by-manoeuvre2copybuffertoimage4rpu to: " + str( max2copybuffertoimagestate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4rpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4rpu = consumerchannel2copybuffertoimagemanoeuvre4rpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4rpu-received-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4rpu == '<copy4xi>copy2copybuffertoimage4rpu</copy4xi>':
                message2copybuffertoimagesend4rpu = '<roger4xi>roger2copybuffertoimage4rpu</roger4xi>'
            else:
                message2copybuffertoimagesend4rpu = '<negative4xi><negative2copybuffertoimage4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4rpu.send(message2copybuffertoimagesend4rpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4rpu-sent-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4rpu-failed-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4rpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4rpu = consumerchannel2copybuffertoimagemanoeuvre4rpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4rpu-received-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4rpu == '<ready4xi>ready2copybuffertoimage4rpu</ready4xi>':
                message2copybuffertoimagesend4rpu = '<ready4xi>ready2copybuffertoimage4rpu</ready4xi>'
            else:
                message2copybuffertoimagesend4rpu = '<negative4xi><negative2copybuffertoimage4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4rpu.send(message2copybuffertoimagesend4rpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4rpu-sent-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4rpu-failed-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4rpu == 3:    
        info2image4rpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4rpu-try-by-manoeuvre2copybuffertoimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4rpu = queue2copybuffertoimagemanoeuvre4rpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4rpu-got-by-manoeuvre2copybuffertoimage4rpu: " + str(info2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4rpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4rpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4rpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4rpu-failed-by-manoeuvre2copybuffertoimage4rpu: " + str(queue2copybuffertoimagemanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4rpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4rpu is True:
                message2copybuffertoimagesend4rpu = '<ack4xi>ack2imageinfo4rpu</ack4xi>'
            else:
                message2copybuffertoimagesend4rpu = '<nack4xi>nack2imageinfo4rpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4rpu-set-by-manoeuvre2copybuffertoimage4rpu to: " + str(message2copybuffertoimagesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4rpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4rpu-failed-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4rpu-calling-by-manoeuvre2copybuffertoimage4rpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4rpu-passing-to-send-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4rpu-failed-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4rpu.send(message2copybuffertoimagesend4rpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4rpu-sent-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4rpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4rpu-failed-by-manoeuvre2copybuffertoimage4rpu: " + str(message2copybuffertoimagesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4RPU']        = ack2copybuffertoimageinfo4rpu
    response2manouvre4copybuffertoimage['CODE2ACK4RPU']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4RPU']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4RPU']     = message2copybuffertoimageget4rpu
    response2manouvre4copybuffertoimage['MESSAGE2PUT4RPU']     = message2copybuffertoimageput4rpu
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4RPU'] = message2copybuffertoimagereceive4rpu
    response2manouvre4copybuffertoimage['MESSAGE2SEND4RPU']    = message2copybuffertoimagesend4rpu
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4RPU'] = state2copybuffertoimage4rpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4rpu: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4RPU': '<roger4xi>roger2copybuffertoimage4rpu</roger4xi>',
    #   'ACK2INFO4RPU': False,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<copy4xi>copy2copybuffertoimage4rpu</copy4xi>',
    #   'CODE2ACK4RPU': ' 0D-1C-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4RPU': '<ready4xi>ready2copybuffertoimage4rpu</ready4xi>',
    #   'ACK2INFO4RPU': False,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2copybuffertoimage4rpu</ready4xi>',
    #   'CODE2ACK4RPU': ' 0D-2C-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4RPU': '<ack4xi>ack2imageinfo4rpu</ack4xi>',
    #   'ACK2INFO4RPU': True,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2copybuffertoimage4rpu</ready4xi>',
    #   'CODE2ACK4RPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'ACK2INFO4RPU': False,
    #   'MESSAGE2GET4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'MESSAGE2RECEIVE4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'CODE2ACK4RPU': ' 0D-',
    #   'MESSAGE2PUT4RPU': '<default4xi>default2copybuffertoimage4rpu</default4xi>',
    #   'CODE2NACK4RPU': ' ',
    #   'STATE2MANOEUVRE4RPU': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4rpu" usage = "control function for read2image4rpu function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4rpu(queue2readmanoeuvre4rpu, consumerchannel2readmanoeuvre4rpu, publisherchannel2readmanoeuvre4rpu):
    '''
    manoeuvres read control and command
    '''
    global state2read4rpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4rpu                                                                                        # MAX FOR STATE
    global ack2readinfo4rpu                                                                                         # ACK FOR IMAGE INFO
    global message2readget4rpu                                 
    global message2readput4rpu
    global message2readreceive4rpu
    global message2readsend4rpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4rpu" 
    method2print = "manoeuvre2readimage4rpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4rpu-started: " + str(manoeuvre2readimage4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4rpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4rpu-to-manoeuvre2readimage4rpu: " + str(queue2readmanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4rpu-to-manoeuvre2readimage4rpu: " + str(consumerchannel2readmanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4rpu-to-manoeuvre2readimage4rpu: " + str(publisherchannel2readmanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4rpu-try-by-manoeuvre2readimage4rpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4rpu is None:
            state2read4rpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4rpu-initialized-by-manoeuvre2readimage4rpu to: " + str(state2read4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4rpu-used-by-manoeuvre2readimage4rpu: " + str(state2read4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4rpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4rpu-initialized-by-manoeuvre2readimage4rpu to: " + str(state2read4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4rpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4rpu-initialized-by-manoeuvre2readimage4rpu to: " + str(ack2readinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4rpu     = '<default4xi>default2read4rpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4rpu-initialized-by-manoeuvre2readimage4rpu to: " + str(message2readget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4rpu</default4xi>
        message2readput4rpu     = '<default4xi>default2read4rpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4rpu-initialized-by-manoeuvre2readimage4rpu to: " + str(message2readput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4rpu</default4xi>
        message2readreceive4rpu = '<default4xi>default2read4rpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4rpu-initialized-by-manoeuvre2readimage4rpu to: " + str(message2readreceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4rpu</default4xi>
        message2readsend4rpu    = '<default4xi>default2read4rpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4rpu-initialized-by-manoeuvre2readimage4rpu to: " + str(message2readsend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4rpu</default4xi>
        max2readstate4rpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4rpu-initialized-by-manoeuvre2readimage4rpu to: " + str( max2readstate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4rpu = state2read4rpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4rpu-added-by-manoeuvre2readimage4rpu: " + str(state2read4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4rpu > max2readstate4rpu:
        state2read4rpu = 0
        message2print = "{(Step 0I/33) set" + "; state2read4rpu-set-by-manoeuvre2readimage4rpu to: " + str(state2read4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4rpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4rpu-set-by-manoeuvre2readimage4rpu to: " + str(ack2readinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4rpu     = '<default4xi>default2read4rpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4rpu-set-by-manoeuvre2readimage4rpu to: " + str(message2readget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4rpu</default4xi>
        message2readput4rpu     = '<default4xi>default2read4rpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4rpu-set-by-manoeuvre2readimage4rpu to: " + str(message2readput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4rpu</default4xi>
        message2readreceive4rpu = '<default4xi>default2read4rpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4rpu-set-by-manoeuvre2readimage4rpu to: " + str(message2readreceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4rpu</default4xi>
        message2readsend4rpu    = '<default4xi>default2read4rpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4rpu-set-by-manoeuvre2readimage4rpu to: " + str(message2readsend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4rpu</default4xi>
        max2readstate4rpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4rpu-set-by-manoeuvre2readimage4rpu to: " + str( max2readstate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4rpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4rpu = consumerchannel2readmanoeuvre4rpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4rpu-received-by-manoeuvre2readimage4rpu: " + str(message2readreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4rpu</copy4xi>
                         
            if message2readreceive4rpu == '<copy4xi>copy2read4rpu</copy4xi>':
                message2readsend4rpu = '<roger4xi>roger2read4rpu</roger4xi>'
            else:
                message2readsend4rpu = '<negative4xi><negative2read4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4rpu.send(message2readsend4rpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4rpu-sent-by-manoeuvre2readimage4rpu: " + str(message2readsend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4rpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4rpu-failed-by-manoeuvre2readimage4rpu: " + str(message2readreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4rpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4rpu = consumerchannel2readmanoeuvre4rpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4rpu-received-by-manoeuvre2readimage4rpu: " + str(message2readreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4rpu</ready4xi> 
                         
            if message2readreceive4rpu == '<ready4xi>ready2read4rpu</ready4xi>':
                message2readsend4rpu = '<ready4xi>ready2read4rpu</ready4xi>'
            else:
                message2readsend4rpu = '<negative4xi><negative2ready4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4rpu.send(message2readsend4rpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4rpu-sent-by-manoeuvre2readimage4rpu: " + str(message2readsend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4rpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4rpu-failed-by-manoeuvre2readimage4rpu: " + str(message2readreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4rpu == 3:    
        info2image4rpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4rpu-try-by-manoeuvre2readimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4rpu = queue2readmanoeuvre4rpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4rpu-got-by-manoeuvre2readimage4rpu: " + str(info2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4rpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4rpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4rpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4rpu-failed-by-manoeuvre2readimage4rpu: " + str(queue2readmanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4rpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4rpu is True:
                message2readsend4rpu = '<ack4xi>ack2imageinfo4rpu</ack4xi>'
            else:
                message2readsend4rpu = '<nack4xi>nack2imageinfo4rpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4rpu-set-by-manoeuvre2readimage4rpu to: " + str(message2readsend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4rpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4rpu-failed-by-manoeuvre2readimage4rpu: " + str(message2readsend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4rpu-calling-by-manoeuvre2readimage4rpu: " + str(publisherchannel2readmanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4rpu-passing-to-send-by-manoeuvre2readimage4rpu: " + str(message2readsend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4rpu-failed-by-manoeuvre2readimage4rpu: " + str(message2readsend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4rpu.send(message2readsend4rpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4rpu-sent-by-manoeuvre2readimage4rpu: " + str(message2readsend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4rpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4rpu-failed-by-manoeuvre2readimage4rpu: " + str(message2readsend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4RPU']        = ack2readinfo4rpu
    response2manouvre4read['CODE2ACK4RPU']        = code2ack
    response2manouvre4read['CODE2NACK4RPU']       = code2nack
    response2manouvre4read['MESSAGE2GET4RPU']     = message2readget4rpu
    response2manouvre4read['MESSAGE2PUT4RPU']     = message2readput4rpu
    response2manouvre4read['MESSAGE2RECEIVE4RPU'] = message2readreceive4rpu
    response2manouvre4read['MESSAGE2SEND4RPU']    = message2readsend4rpu
    response2manouvre4read['STATE2MANOEUVRE4RPU'] = state2read4rpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4rpu: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4RPU': '<roger4xi>roger2read4rpu</roger4xi>',
    #  'ACK2INFO4RPU': False,
    #  'MESSAGE2GET4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'MESSAGE2RECEIVE4RPU': '<copy4xi>copy2read4rpu</copy4xi>',
    #  'CODE2ACK4RPU': ' 0D-1C-',
    #  'MESSAGE2PUT4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'CODE2NACK4RPU': ' ', 'STATE2MANOEUVRE4RPU': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4RPU': '<ready4xi>ready2read4rpu</ready4xi>',
    #  'ACK2INFO4RPU': False,
    #  'MESSAGE2GET4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2read4rpu</ready4xi>',
    #  'CODE2ACK4RPU': ' 0D-2C-',
    #  'MESSAGE2PUT4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'CODE2NACK4RPU': ' ', 'STATE2MANOEUVRE4RPU': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4RPU': '<ack4xi>ack2imageinfo4rpu</ack4xi>',
    #  'ACK2INFO4RPU': True,
    #  'MESSAGE2GET4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2read4rpu</ready4xi>',
    #  'CODE2ACK4RPU': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'CODE2NACK4RPU': ' ',
    #  'STATE2MANOEUVRE4RPU': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'ACK2INFO4RPU': False,
    #  'MESSAGE2GET4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'MESSAGE2RECEIVE4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'CODE2ACK4RPU': ' 0D-',
    #  'MESSAGE2PUT4RPU': '<default4xi>default2read4rpu</default4xi>',
    #  'CODE2NACK4RPU': ' ',
    #  'STATE2MANOEUVRE4RPU': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4rpu" usage = "control function for write2image4rpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4rpu(queue2writemanoeuvre4rpu, consumerchannel2writemanoeuvre4rpu, publisherchannel2writemanoeuvre4rpu):
    '''
    manoeuvres control and command
    '''
    global state2write4rpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4rpu                                                    # MAX FOR STATE
    global ack2writeinfo4rpu                                                     # ACK FOR IMAGE INFO
    global message2writeget4rpu                                 
    global message2writeput4rpu
    global message2writereceive4rpu
    global message2writesend4rpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4rpu" 
    method2print = "manoeuvre2writeimage4rpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4rpu-started: " + str(manoeuvre2writeimage4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4rpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4rpu-to-manoeuvre2writeimage4rpu: " + str(queue2writemanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4rpu-to-manoeuvre2writeimage4rpu: " + str(consumerchannel2writemanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4rpu-to-manoeuvre2writeimage4rpu: " + str(publisherchannel2writemanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4rpu-try-by-manoeuvre2writeimage4rpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4rpu is None:
            state2write4rpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4rpu-initialized-by-manoeuvre2writeimage4rpu to: " + str(state2write4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4rpu-used-by-manoeuvre2writeimage4rpu: " + str(state2write4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4rpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4rpu-initialized-by-manoeuvre2writeimage4rpu to: " + str(state2write4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4rpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4rpu-initialized-by-manoeuvre2writeimage4rpu to: " + str(ack2writeinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4rpu     = '<default4xi>default2write4rpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4rpu-initialized-by-manoeuvre2writeimage4rpu to: " + str(message2writeget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4rpu</default4xi>
        message2writeput4rpu     = '<default4xi>default2write4rpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4rpu-initialized-by-manoeuvre2writeimage4rpu to: " + str(message2writeput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4rpu</default4xi>
        message2writereceive4rpu = '<default4xi>default2write4rpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4rpu-initialized-by-manoeuvre2writeimage4rpu to: " + str(message2writereceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4rpu</default4xi>
        message2writesend4rpu    = '<default4xi>default2write4rpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4rpu-initialized-by-manoeuvre2writeimage4rpu to: " + str(message2writesend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4rpu</default4xi>
        max2writestate4rpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4rpu-initialized-by-manoeuvre2writeimage4rpu to: " + str( max2writestate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4rpu = state2write4rpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4rpu-added-by-manoeuvre2writeimage4rpu: " + str(state2write4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4rpu > max2writestate4rpu:
        state2write4rpu = 0
        message2print = "{(Step 0I/33) set" + "; state2write4rpu-set-by-manoeuvre2writeimage4rpu to: " + str(state2write4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4rpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4rpu-set-by-manoeuvre2writeimage4rpu to: " + str(ack2writeinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4rpu     = '<default4xi>default2write4rpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4rpu-set-by-manoeuvre2writeimage4rpu to: " + str(message2writeget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4rpu</default4xi>
        message2writeput4rpu     = '<default4xi>default2write4rpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4rpu-set-by-manoeuvre2writeimage4rpu to: " + str(message2writeput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4rpu</default4xi>
        message2writereceive4rpu = '<default4xi>default2write4rpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4rpu-set-by-manoeuvre2writeimage4rpu to: " + str(message2writereceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4rpu</default4xi>
        message2writesend4rpu    = '<default4xi>default2write4rpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4rpu-set-by-manoeuvre2writeimage4rpu to: " + str(message2writesend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4rpu</default4xi>
        max2writestate4rpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4rpu-set-by-manoeuvre2writeimage4rpu to: " + str( max2writestate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4rpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4rpu = consumerchannel2writemanoeuvre4rpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4rpu-received-by-manoeuvre2writeimage4rpu: " + str(message2writereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4rpu</copy4xi>
                         
            if message2writereceive4rpu == '<copy4xi>copy2write4rpu</copy4xi>':
                message2writesend4rpu = '<roger4xi>roger2write4rpu</roger4xi>'
            else:
                message2writesend4rpu = '<negative4xi><negative2write4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4rpu.send(message2writesend4rpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4rpu-sent-by-manoeuvre2writeimage4rpu: " + str(message2writesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4rpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4rpu-failed-by-manoeuvre2writeimage4rpu: " + str(message2writereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4rpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4rpu = consumerchannel2writemanoeuvre4rpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4rpu-received-by-manoeuvre2writeimage4rpu: " + str(message2writereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4rpu</ready4xi>
                         
            if message2writereceive4rpu == '<ready4xi>ready2write4rpu</ready4xi>':
                message2writesend4rpu = '<ready4xi>ready2write4rpu</ready4xi>'
            else:
                message2writesend4rpu = '<negative4xi><negative2ready4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4rpu.send(message2writesend4rpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4rpu-sent-by-manoeuvre2writeimage4rpu: " + str(message2writesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4rpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4rpu-failed-by-manoeuvre2writeimage4rpu: " + str(message2writereceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4rpu == 3:    
        info2image4rpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4rpu-try-by-manoeuvre2writeimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4rpu = queue2writemanoeuvre4rpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4rpu-got-by-manoeuvre2writeimage4rpu: " + str(info2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4rpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4rpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4rpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4rpu-failed-by-manoeuvre2writeimage4rpu: " + str(queue2writemanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4rpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4rpu is True:
                message2writesend4rpu = '<ack4xi>ack2imageinfo4rpu</ack4xi>'
            else:
                message2writesend4rpu = '<nack4xi>nack2imageinfo4rpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4rpu-set-by-manoeuvre2writeimage4rpu to: " + str(message2writesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4rpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4rpu-failed-by-manoeuvre2writeimage4rpu: " + str(message2writesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4rpu-calling-by-manoeuvre2writeimage4rpu: " + str(publisherchannel2writemanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4rpu-passing-to-send-by-manoeuvre2writeimage4rpu: " + str(message2writesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4rpu-failed-by-manoeuvre2writeimage4rpu: " + str(message2writesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4rpu.send(message2writesend4rpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4rpu-sent-by-manoeuvre2writeimage4rpu: " + str(message2writesend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4rpu</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4rpu-failed-by-manoeuvre2writeimage4rpu: " + str(message2writesend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4RPU']        = ack2writeinfo4rpu
    response2manouvre4write['CODE2ACK4RPU']        = code2ack
    response2manouvre4write['CODE2NACK4RPU']       = code2nack
    response2manouvre4write['MESSAGE2GET4RPU']     = message2writeget4rpu
    response2manouvre4write['MESSAGE2PUT4RPU']     = message2writeput4rpu
    response2manouvre4write['MESSAGE2RECEIVE4RPU'] = message2writereceive4rpu
    response2manouvre4write['MESSAGE2SEND4RPU']    = message2writesend4rpu
    response2manouvre4write['STATE2MANOEUVRE4RPU'] = state2write4rpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4rpu: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4RPU': '<roger4xi>roger2write4rpu</roger4xi>', 
    #  'ACK2INFO4RPU': False, 'MESSAGE2GET4RPU': '<default4xi>default2write4rpu</default4xi>', 
    #  'MESSAGE2RECEIVE4RPU': '<copy4xi>copy2write4rpu</copy4xi>', 
    #  'CODE2ACK4RPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4RPU': '<default4xi>default2write4rpu</default4xi>', 
    #  'CODE2NACK4RPU': ' ', 
    #  'STATE2MANOEUVRE4RPU': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4RPU': '<ready4xi>ready2write4rpu</ready4xi>', 
    #  'ACK2INFO4RPU': False, 'MESSAGE2GET4RPU': '<default4xi>default2write4rpu</default4xi>', 
    #  'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2write4rpu</ready4xi>', 
    #  'CODE2ACK4RPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4RPU': '<default4xi>default2write4rpu</default4xi>', 
    #  'CODE2NACK4RPU': ' ', 
    #  'STATE2MANOEUVRE4RPU': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4RPU': '<ack4xi>ack2imageinfo4rpu</ack4xi>', 
    #  'ACK2INFO4RPU': True, 'MESSAGE2GET4RPU': '<default4xi>default2write4rpu</default4xi>', 
    #  'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2write4rpu</ready4xi>', 
    #  'CODE2ACK4RPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4RPU': '<default4xi>default2write4rpu</default4xi>', 
    #  'CODE2NACK4RPU': ' ', 
    #  'STATE2MANOEUVRE4RPU': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4rpu" usage = "control function for map2image4rpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4rpu(queue2mapmanoeuvre4rpu, consumerchannel2mapmanoeuvre4rpu, publisherchannel2mapmanoeuvre4rpu):
    '''
    manoeuvres control and command
    '''
    global state2map4rpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4rpu                                                    # MAX FOR STATE
    global ack2mapinfo4rpu                                                     # ACK FOR IMAGE INFO
    global message2mapget4rpu                                 
    global message2mapput4rpu
    global message2mapreceive4rpu
    global message2mapsend4rpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4rpu" 
    method2print = "manoeuvre2mapimage4rpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4rpu-started: " + str(manoeuvre2mapimage4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4rpu-to-manoeuvre2mapimage4rpu: " + str(queue2mapmanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4rpu-to-manoeuvre2mapimage4rpu: " + str(consumerchannel2mapmanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4rpu-to-manoeuvre2mapimage4rpu: " + str(publisherchannel2mapmanoeuvre4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4rpu-try-by-manoeuvre2mapimage4rpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4rpu is None:
            state2map4rpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4rpu-initialized-by-manoeuvre2mapimage4rpu to: " + str(state2map4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4rpu-used-by-manoeuvre2mapimage4rpu: " + str(state2map4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4rpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4rpu-initialized-by-manoeuvre2mapimage4rpu to: " + str(state2map4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4rpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4rpu-initialized-by-manoeuvre2mapimage4rpu to: " + str(ack2mapinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4rpu     = '<default4xi>default2map4rpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4rpu-initialized-by-manoeuvre2mapimage4rpu to: " + str(message2mapget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4rpu</default4xi>
        message2mapput4rpu     = '<default4xi>default2map4rpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4rpu-initialized-by-manoeuvre2mapimage4rpu to: " + str(message2mapput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4rpu</default4xi>
        message2mapreceive4rpu = '<default4xi>default2map4rpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4rpu-initialized-by-manoeuvre2mapimage4rpu to: " + str(message2mapreceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4rpu</default4xi>
        message2mapsend4rpu    = '<default4xi>default2map4rpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4rpu-initialized-by-manoeuvre2mapimage4rpu to: " + str(message2mapsend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4rpu</default4xi>
        max2mapstate4rpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4rpu-initialized-by-manoeuvre2mapimage4rpu to: " + str( max2mapstate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4rpu = state2map4rpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4rpu-added-by-manoeuvre2mapimage4rpu: " + str(state2map4rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4rpu > max2mapstate4rpu:
        state2map4rpu = 0
        message2print = "{(Step 0I/33) set" + "; state2map4rpu-set-by-manoeuvre2mapimage4rpu to: " + str(state2map4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4rpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4rpu-set-by-manoeuvre2mapimage4rpu to: " + str(ack2mapinfo4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4rpu     = '<default4xi>default2map4rpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4rpu-set-by-manoeuvre2mapimage4rpu to: " + str(message2mapget4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4rpu</default4xi>
        message2mapput4rpu     = '<default4xi>default2map4rpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4rpu-set-by-manoeuvre2mapimage4rpu to: " + str(message2mapput4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4rpu</default4xi>
        message2mapreceive4rpu = '<default4xi>default2map4rpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4rpu-set-by-manoeuvre2mapimage4rpu to: " + str(message2mapreceive4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4rpu</default4xi>
        message2mapsend4rpu    = '<default4xi>default2map4rpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4rpu-set-by-manoeuvre2mapimage4rpu to: " + str(message2mapsend4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4rpu</default4xi>
        max2mapstate4rpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4rpu-set-by-manoeuvre2mapimage4rpu to: " + str( max2mapstate4rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4rpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4rpu = consumerchannel2mapmanoeuvre4rpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4rpu-received-by-manoeuvre2mapimage4rpu: " + str(message2mapreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4rpu == '<copy4xi>copy2mapimage4rpu</copy4xi>':
                message2mapsend4rpu = '<roger4xi>roger2mapimage4rpu</roger4xi>'
            else:
                message2mapsend4rpu = '<negative4xi><negative2mapimage4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4rpu.send(message2mapsend4rpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4rpu-sent-by-manoeuvre2mapimage4rpu: " + str(message2mapsend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4rpu-failed-by-manoeuvre2mapimage4rpu: " + str(message2mapreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4rpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4rpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4rpu = consumerchannel2mapmanoeuvre4rpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4rpu-received-by-manoeuvre2mapimage4rpu: " + str(message2mapreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4rpu == '<ready4xi>ready2mapimage4rpu</ready4xi>':
                message2mapsend4rpu = '<ready4xi>ready2mapimage4rpu</ready4xi>'
            else:
                message2mapsend4rpu = '<negative4xi><negative2ready4rpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4rpu.send(message2mapsend4rpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4rpu-sent-by-manoeuvre2mapimage4rpu: " + str(message2mapsend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4rpu-failed-by-manoeuvre2mapimage4rpu: " + str(message2mapreceive4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4rpu == 3:    
        info2image4rpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4rpu-try-by-manoeuvre2mapimage4rpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4rpu = queue2mapmanoeuvre4rpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4rpu-got-by-manoeuvre2mapimage4rpu: " + str(info2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4rpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4rpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4rpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4rpu-failed-by-manoeuvre2mapimage4rpu: " + str(queue2mapmanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4rpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4rpu is True:
                message2mapsend4rpu = '<ack4xi>ack2imageinfo4rpu</ack4xi>'
            else:
                message2mapsend4rpu = '<nack4xi>nack2imageinfo4rpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4rpu-set-by-manoeuvre2mapimage4rpu to: " + str(message2mapsend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4rpu-failed-by-manoeuvre2mapimage4rpu: " + str(message2mapsend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4rpu-calling-by-manoeuvre2mapimage4rpu: " + str(publisherchannel2mapmanoeuvre4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4rpu-passing-to-send-by-manoeuvre2mapimage4rpu: " + str(message2mapsend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4rpu-failed-by-manoeuvre2mapimage4rpu: " + str(message2mapsend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4rpu.send(message2mapsend4rpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4rpu-sent-by-manoeuvre2mapimage4rpu: " + str(message2mapsend4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4rpu-failed-by-manoeuvre2mapimage4rpu: " + str(message2mapsend4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4RPU']        = ack2mapinfo4rpu
    response2manouvre4map['CODE2ACK4RPU']        = code2ack
    response2manouvre4map['CODE2NACK4RPU']       = code2nack
    response2manouvre4map['MESSAGE2GET4RPU']     = message2mapget4rpu
    response2manouvre4map['MESSAGE2PUT4RPU']     = message2mapput4rpu
    response2manouvre4map['MESSAGE2RECEIVE4RPU'] = message2mapreceive4rpu
    response2manouvre4map['MESSAGE2SEND4RPU']    = message2mapsend4rpu
    response2manouvre4map['STATE2MANOEUVRE4RPU'] = state2map4rpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4rpu: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4RPU': '<roger4xi>roger2mapimage4rpu</roger4xi>', 
    #  'ACK2INFO4RPU': False, 
    #  'MESSAGE2GET4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'MESSAGE2RECEIVE4RPU': '<copy4xi>copy2mapimage4rpu</copy4xi>', 
    #  'CODE2ACK4RPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'CODE2NACK4RPU': ' ', 
    #  'STATE2MANOEUVRE4RPU': 1}
     
    #2 {'MESSAGE2SEND4RPU': '<ready4xi>ready2mapimage4rpu</ready4xi>', 
    #  'ACK2INFO4RPU': False, 
    #  'MESSAGE2GET4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2mapimage4rpu</ready4xi>', 
    #  'CODE2ACK4RPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'CODE2NACK4RPU': ' ', 
    #  'STATE2MANOEUVRE4RPU': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4RPU': '<ack4xi>ack2imageinfo4rpu</ack4xi>', 
    #  'ACK2INFO4RPU': True, 
    #  'MESSAGE2GET4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'MESSAGE2RECEIVE4RPU': '<ready4xi>ready2mapimage4rpu</ready4xi>', 
    #  'CODE2ACK4RPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'CODE2NACK4RPU': ' ', 
    #  'STATE2MANOEUVRE4RPU': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'ACK2INFO4RPU': False, 
    #  'MESSAGE2GET4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'MESSAGE2RECEIVE4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'CODE2ACK4RPU': ' 0D-', 
    #  'MESSAGE2PUT4RPU': '<default4xi>default2map4rpu</default4xi>', 
    #  'CODE2NACK4RPU': ' ', 'STATE2MANOEUVRE4RPU': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4rpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4RPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4RPU ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4rpu(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4RPU : ', response4write2image)
#       queue2write4rpu      = response4write2image['QUEUE']
#       channel2publish4rpu = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4rpu = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4RPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4RPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())     
#   try:                               
#       message2writesend4rpu = "<copy4xi>copy2write4rpu</copy4xi>"
#       channel2publish4rpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4rpu.send(message2writesend4rpu)
#       print("(Step 17B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2publish4rpu, " MESSAGE2SEND4RPU: " , message2writesend4rpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4RPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2readimage4rpu(queue2write4rpu, channel2consume4rpu, channel2publish4rpu)
#       print("(Step 18D/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", manoeuvre2readimage4rpu)
#   except:
#       print("(Step 18Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())     
#   try:
#       channel2consume4rpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4rpu = channel2consume4rpu.recv()
#       print("(Step 19B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2consume4rpu, " CONTROL & COMMAND: ", message2writereceive4rpu)                                                                                                  
#       # <roger4xi>roger2write4rpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4RPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4RPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF RPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())     
#   try:                               
#       message2writesend4rpu = "<write4xi>write2write4rpu</write4xi>"
#       channel2publish4rpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4rpu.send(message2writesend4rpu)
#       print("(Step 20B/700) OF RPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4RPU: ", channel2publish4rpu, " MESSAGE2SEND4RPU: " , message2writesend4rpu)                                                                                                
#       # <write4xi>write2write4rpu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4RPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2readimage4rpu(queue2write4rpu, channel2consume4rpu, channel2publish4rpu)
#       print("(Step 21B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", manoeuvre2readimage4rpu)
#   except:
#       print("(Step 21Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4RPU ', time4xi())     
#   try:
#       channel2consume4rpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4rpu = channel2consume4rpu.recv()
#       print("(Step 22B/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2consume4rpu, " CONTROL & COMMAND: ", message2writereceive4rpu)                                                                                                  
#       # <write4xi>write2write4rpu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4RPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4RPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2readimage4rpu(queue2write4rpu, channel2consume4rpu, channel2publish4rpu)
#       print("(Step 25B/700) OF RPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", manoeuvre2readimage4rpu)
#   except:
#       print("(Step 25Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4rpu = channel2consume4rpu.recv()
#       print("(Step 26D/700) OF RPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4RPU: ", channel2consume4rpu, " CONTROL & COMMAND: ", message2writereceive4rpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4rpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4RPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF RPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi()) 
#   try:        
#       response2manoeuvre4rpu = manoeuvre2readimage4rpu(queue2write4rpu, channel2consume4rpu, channel2publish4rpu)
#       print("(Step 27B/700) OF RPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4RPU ", response2manoeuvre4rpu)
#   except:
#       print("(Step 27Z/700) OF RPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4RPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4rpu(task4imagewrite2rpu=None):

    '''
    writes 2D images for RPU
    '''
    global rpu2image4xi                                                     # 2D-IMAGES FOR RPU  
    global control2writeimage4rpu                                              # COMMAND AND CONTROL FOR RPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4rpu"
    method2print = "write2image4rpu"

    message2print = "{(Step 1A/33) started" + "; write2image4rpu-started: " + str(write2image4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4rpu at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2rpu-to-write2image4rpu: " + str(task4imagewrite2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE RPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if rpu2image4xi is None:
            rpu2image4xi = create2image4rpu()                                                                              # INITIALIZE IMAGE FOR RPU 
            message2print = "{(Step 1C/70) created" + "; rpu2image4xi-created-by-write2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4rpu-skipped-by-write2image4rpu: " + str(write2image4rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4rpu-skipped-by-write2image4rpu: <function write2image4rpu at 0x00000000038E8EC8>                                 
    except:
        try:
            rpu2image4xi = create2image4rpu()
            message2print = "{(Step 1E/70) set" + "; rpu2image4xi-set-by-write2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            rpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; rpu2image4xi-yielded-by-write2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2rpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2rpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2rpu-created-by-write2image4rpu to: " + str(task4imagewrite2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4rpu = trace2image4rpu                                                                 # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4rpu-created-by-write2image4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4rpu at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4rpu"                                                            # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4rpu"                                                            # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4rpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4rpu', 
            #  'METHOD': 'write2image4rpu'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2rpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2rpu-created-by-write2image4rpu to: " + str(task4imagewrite2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4rpu at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4rpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4rpu',
            #  'METHOD': 'write2image4rpu'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2rpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2rpu-trying-by-write2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2rpu-got-by-write2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4rpu', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2rpu-failed-by-write2image4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2rpu ['CONTEXT'] = context2rpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2rpu-trying-by-write2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2rpu-got-by-write2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2rpu-failed-by-write2image4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2rpu ['DEVICE'] = device2rpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2rpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2rpu-trying-by-write2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2rpu-got-by-write2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4rpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4rpu', 
                #  'METHOD': 'create2context4rpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4rpu at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4rpu at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2rpu-failed-by-write2image4rpu: " + str(imageinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2rpu ['IMAGE'] = imageinfo2rpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY RPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2rpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2rpu-trying-by-write2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2rpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2rpu-got-by-write2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2rpu-failed-by-write2image4rpu: " + str(queue2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2rpu ['QUEUE'] = queue2rpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY RPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2rpu-trying-by-write2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2rpu-got-by-write2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2rpu-failed-by-write2image4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2rpu ['PIPE'] = pipe2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2rpu-trying-by-write2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2rpu-got-by-write2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4rpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4rpu', 
                #  'METHOD': 'create2commandqueue4rpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4rpu at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2rpu-failed-by-write2image4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY RPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2rpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2rpu-trying-by-write2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2rpu = get2bufferinfo4rpu()                                                               # SYSTEM CREATES BUFFER FOR RPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2rpu-got-by-write2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4rpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4rpu', 
                #  'METHOD': 'create2buffer4rpu'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2rpu-failed-by-write2image4rpu: " + str(bufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2rpu ['BUFFER'] = bufferinfo2rpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2rpu-trying-by-write2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2rpu-got-by-write2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4rpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4rpu', 
                #  'METHOD': 'create2event4rpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4rpu at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2rpu-failed-by-write2image4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2rpu ['EVENT'] = eventinfo2rpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2rpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2rpu-trying-by-write2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2rpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2rpu-set-by-write2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2rpu-failed-by-write2image4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2rpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-write2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2rpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2rpu-set-by-write2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-write2image4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2rpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2rpu-trying-by-write2image4rpu: " + str(blocking2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2rpu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2rpu-set-by-write2image4rpu to: " + str(blocking2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2rpu-failed-by-write2image4rpu: " + str(blocking2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2rpu ['BLOCKING'] = blocking2rpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2rpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2rpu-trying-by-write2image4rpu: " + str(origin2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2rpu['X'] = 0 
                origin2rpu['Y'] = 0  
                origin2rpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2rpu-set-by-write2image4rpu to: " + str(origin2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2rpu-failed-by-write2image4rpu: " + str(origin2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2rpu ['ORIGIN'] = origin2rpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2rpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2rpu-trying-by-write2image4rpu: " + str(region2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2rpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2rpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2rpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2rpu-set-by-write2image4rpu to: " + str(region2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2rpu-failed-by-write2image4rpu: " + str(region2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2rpu ['REGION'] = region2rpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2rpu = 800
            width2rpu = 800
            rowpitch2rpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2rpu-trying-by-write2image4rpu: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2rpu = int(width2rpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2rpu-set-by-write2image4rpu to: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2rpu-failed-by-write2image4rpu: " + str(rowpitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2rpu ['ROWPITCH'] = rowpitch2rpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2rpu = None
            size2rpu = int(width2rpu * height2rpu)                                                                  # CALCULATED MEMORY
            if size2rpu == 0:
                size2rpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2rpu-trying-by-write2image4rpu: " + str(pointer2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2rpu = Arena4Xi(size2rpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2rpu-got-by-write2image4rpu: " + str(arena2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2rpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4rpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2rpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4rpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2rpu:
                    size2rpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2rpu-set-by-write2buffer4rpu to: " + str(size2rpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2rpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2rpu-set-by-write2buffer4rpu to: " + str(pointer2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2rpu-failed-by-write2image4rpu: " + str(pointer2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2rpu ['POINTER'] = pointer2rpu                            
            task4imagewrite2rpu ['SIZE']    = size2rpu                                                               # AUTOMATIC BY RPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2rpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2rpu-trying-by-write2image4rpu: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2rpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2rpu-set-by-write2image4rpu to: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2rpu-failed-by-write2image4rpu: " + str(slicepitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2rpu ['SLICEPITCH'] = slicepitch2rpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2rpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4rpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4rpu = trace2image4rpu                                                             # CALLBACK FUNCTION FOR RPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4rpu-created-by-write2image4rpu to: " + str(function2callback4rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4rpu"                                                        # MODULE OF RPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4rpu"                                                        # FUNCTION OF RPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4rpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR RPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4rpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2rpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO RPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2rpu['FUNCTION2CALLBACK'] = function2callback4rpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2rpu-created-by-write2image4rpu to: " + str(task4imagewrite2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4rpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4rpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4rpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2rpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2rpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2rpu-trying-by-write2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2rpu = get2context4rpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2rpu-got-by-write2image4rpu: " + str(context2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2rpu-failed-by-write2image4rpu: " + str(context2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2rpu ['CONTEXT'] = context2rpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2rpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2rpu-trying-by-write2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2rpu = get2rpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2rpu-got-by-write2image4rpu: " + str(device2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2rpu-failed-by-write2image4rpu: " + str(device2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2rpu ['DEVICE'] = device2rpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2rpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2rpu-trying-by-write2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2rpu = get2imageinfo4rpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2rpu-got-by-write2image4rpu: " + str(imageinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2rpu-failed-by-write2image4rpu: " + str(imageinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2rpu ['IMAGE'] = imageinfo2rpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY RPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2rpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2rpu-trying-by-write2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2rpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2rpu-got-by-write2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2rpu-failed-by-write2image4rpu: " + str(queue2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2rpu ['QUEUE'] = queue2rpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY RPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2rpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2rpu-trying-by-write2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2rpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2rpu-got-by-write2image4rpu: " + str(pipe2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2rpu-failed-by-write2image4rpu: " + str(pipe2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2rpu ['PIPE'] = pipe2rpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2rpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2rpu-trying-by-write2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2rpu = get2commandqueueinfo4rpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2rpu-got-by-write2image4rpu: " + str(commandqueueinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2rpu-failed-by-write2image4rpu: " + str(commandqueueinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2rpu ['COMMANDQUEUE'] = commandqueueinfo2rpu
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY RPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2rpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2rpu-trying-by-write2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2rpu = get2bufferinfo4rpu()                                                           # SYSTEM CREATES BUFFER FOR RPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2rpu-got-by-write2image4rpu: " + str(bufferinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2rpu-failed-by-write2image4rpu: " + str(bufferinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2rpu ['BUFFER'] = bufferinfo2rpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2rpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2rpu-trying-by-write2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2rpu = get2eventinfo4rpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2rpu-got-by-write2image4rpu: " + str(eventinfo2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2rpu-failed-by-write2image4rpu: " + str(eventinfo2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2rpu ['EVENT'] = eventinfo2rpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2rpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2rpu-trying-by-write2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2rpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2rpu-set-by-write2image4rpu: " + str(eventwaitlist2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2rpu-failed-by-write2image4rpu: " + str(eventwaitlist2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2rpu ['EVENTWAITLIST'] = eventwaitlist2rpu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2rpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2rpu-trying-by-write2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2rpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2rpu = eventwaitlistquantity2rpu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2rpu-set-by-write2image4rpu: " + str(eventwaitlistquantity2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2rpu-failed-by-write2image4rpu: " + str(eventwaitlistquantity2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2rpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2rpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2rpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2rpu-trying-by-write2image4rpu: " + str(blocking2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2rpu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2rpu-set-by-write2image4rpu to: " + str(blocking2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2rpu-failed-by-write2image4rpu: " + str(blocking2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2rpu ['BLOCKING'] = blocking2rpu             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2rpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2rpu-trying-by-write2image4rpu: " + str(origin2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2rpu['X'] = 0 
                origin2rpu['Y'] = 0  
                origin2rpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2rpu-set-by-write2image4rpu to: " + str(origin2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2rpu-failed-by-write2image4rpu: " + str(origin2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2rpu ['ORIGIN'] = origin2rpu
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2rpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2rpu-trying-by-write2image4rpu: " + str(region2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2rpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2rpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2rpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2rpu-set-by-write2image4rpu to: " + str(region2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2rpu-failed-by-write2image4rpu: " + str(region2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2rpu ['REGION'] = region2rpu
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2rpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2rpu-trying-by-write2image4rpu: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2rpu = int(width2rpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2rpu-set-by-write2image4rpu to: " + str(rowpitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2rpu-failed-by-write2image4rpu: " + str(rowpitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2rpu ['ROWPITCH'] = rowpitch2rpu     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2rpu = None
            size2rpu = int(width2rpu * height2rpu)                                                                  # CALCULATED MEMORY
            if size2rpu == 0:
                size2rpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2rpu-trying-by-write2image4rpu: " + str(pointer2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2rpu = Arena4Xi(size2rpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2rpu-got-by-write2image4rpu: " + str(arena2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2rpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4rpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2rpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4rpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2rpu:
                    size2rpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2rpu-set-by-write2buffer4rpu to: " + str(size2rpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2rpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2rpu-set-by-write2buffer4rpu to: " + str(pointer2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2rpu-failed-by-write2image4rpu: " + str(pointer2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2rpu ['POINTER'] = pointer2rpu                            
            task4imagewrite2rpu ['SIZE']    = size2rpu                                                               # AUTOMATIC BY RPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2rpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2rpu-trying-by-write2image4rpu: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2rpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2rpu-set-by-write2image4rpu to: " + str(slicepitch2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2rpu-failed-by-write2image4rpu: " + str(slicepitch2rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2rpu ['SLICEPITCH'] = slicepitch2rpu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2rpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4rpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2rpu-yielded-by-write2image4rpu: " + str(task4imagewrite2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4rpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2rpu = False
    channels2rpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2rpu-trying-by-write2image4rpu: " + str(channels2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2rpu, publisherchannel2rpu = task4imagewrite2rpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2rpu-got-by-write2image4rpu: " + str(consumerchannel2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2rpu-got-by-write2image4rpu: " + str(publisherchannel2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2rpu = True
        
        control2writeimage4rpu['CONSUMERCHANNEL']  = consumerchannel2rpu
        
        
        control2writeimage4rpu['PUBLISHERCHANNEL'] = publisherchannel2rpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2rpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2rpu = False
    queue2rpu = None
    message2print = "{(Step 41A/70) trying" + "queue2rpu-trying-by-write2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2rpu = task4imagewrite2rpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2rpu-got-by-write2image4rpu: " + str(queue2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2rpu = True
        control2writeimage4rpu['QUEUE']  = queue2rpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2rpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4rpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4rpu-trying-by-write2image4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4rpu = DummyProcess4Xi(target=manoeuvre2readimage4rpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2rpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2rpu, publisherchannel2rpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4rpu-created-by-write2image4rpu: " + str(dummy2image4rpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4rpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4rpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4rpu-returning-by-write2image4rpu: " + str(control2writeimage4rpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4rpu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4rpu" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF RPU2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF RPU DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4rpu(task4release2image)                                                                                             
#       print('(Step 12B/33) OF RPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF RPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF RPU DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4rpu(task4release2rpu=None):

    '''
    releases rpu2image4xi
    '''
    global rpu2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4rpu"
    method2print = "release2image4rpu" 

    message2print = "{(Step 1A/33) started" + "; release2image4rpu-started: " + str(release2image4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4rpu at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2rpu-to-release2image4rpu: " + str(task4release2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if rpu2image4xi is None:
            rpu2image4xi = create2image4rpu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            rpu2image4xi = create2image4rpu()
        except:
            rpu2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; rpu2image4xi-yielded-by-release2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4rpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2rpu is None:
        task4release2rpu = {}
        task4release2rpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        rpu2image4xi['COUNT2REFERENCE'] = rpu2image4xi['COUNT2REFERENCE'] - task4release2rpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4rpu: " + str(rpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if rpu2image4xi['COUNT2REFERENCE'] < 0:
            rpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4rpu: " + " ** " + str(rpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4rpu : release2image4rpu ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4rpu:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            rpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4rpu: " + " ** " + str(rpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; rpu2image4xi-yielded-by-release2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    rpu2image4xi ['ACK2RETURN']  = code2ack
    rpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; rpu2image4xi-returning-by-release2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4rpu',
    #  'METHOD': 'create2image4rpu'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4rpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4rpu', 'METHOD': 'create2buffer4rpu'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4rpu at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return rpu2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4rpu" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF RPU DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4rpu(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF RPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF RPU DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4rpu(task4retain2rpu=None):

    '''
    retains either rpu2image4xi
    '''
    global rpu2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4rpu"
    method2print = "retain2image4rpu"

    message2print = "{(Step 1A/33) started" + "; retain2image4rpu-started: " + str(retain2image4rpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4rpu at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2rpu-to-retain2image4rpu: " + str(task4retain2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2rpu is None:
        task4retain2rpu = {}
        task4retain2rpu['DIMENSION'] = 2
        task4retain2rpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2rpu-set-by-retain2image4rpu to: " + str(task4retain2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2rpu.keys():
        if task4retain2rpu['DIMENSION'] == 2:
            pass
        else:
            task4retain2rpu['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4rpu to: " + str(task4retain2rpu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2rpu['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2rpu.keys():
        pass
    else:
        task4retain2rpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4rpu to: " + str(task4retain2rpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2rpu['COUNT2REFERENCE'] < 1:
        task4retain2rpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4rpu to: " + str(task4retain2rpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if rpu2image4xi is None:
            rpu2image4xi = create2image4rpu()
            message2print = "{(Step 4A/70) created" + "; rpu2image4xi-created-by-retain2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            rpu2image4xi = create2image4rpu()
            message2print = "{(Step 4B/70) created" + "; rpu2image4xi-created-by-retain2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            rpu2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; rpu2image4xi-yielded-by-retain2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        rpu2image4xi['COUNT2REFERENCE'] = rpu2image4xi['COUNT2REFERENCE'] + task4retain2rpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4rpu: " + str(rpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if rpu2image4xi['COUNT2REFERENCE'] <= 0:
            rpu2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4rpu: " + " ** " + str(rpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            rpu2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4rpu: " + " ** " + str(rpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; rpu2image4xi-yielded-by-retain2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    rpu2image4xi ['ACK2RETURN']  = code2ack
    rpu2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; rpu2image4xi-returning-by-retain2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4rpu', 'METHOD': 'create2image4rpu'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return rpu2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4rpu" usage = "
#   #1
#   print('(Step 3A/33) OF RPU2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR RPU', time4xi())  
#   try:
#       imageinfo2rpu = get2imageinfo4rpu()                                                                       
#       print('(Step 3B/33) OF RPU2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR RPU: ', imageinfo2rpu) 
#   except:
#       print('(Step 3Z/33) OF RPU2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR RPU', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF RPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF RPU DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4rpu (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF RPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF RPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF RPU DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4rpu(task4getinfo2image=None):

    '''
    gets rpu2image4xi
    '''
    
    global rpu2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4rpu"
    method2print = "get2imageinfo4rpu"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4rpu-started: " + str(get2imageinfo4rpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4rpu at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4rpu: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2rpu = {}
    try: 
        if rpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR RPU           
            # -------------------------------------------------------------------------------------           
            image2rpu = create2image4rpu()
            message2print = "{(Step 1B2/33) initialized" + "; image2rpu-initialized-by-get2imageinfo4rpu to: " + str(image2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; rpu2image4xi-using-by-get2imageinfo4rpu: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4rpu', 
            #  'METHOD': 'create2image4rpu'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF RPU          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2rpu = rpu2image4xi
            message2print = "{(Step 1C/33) using" + "; image2rpu-got-by-get2imageinfo4rpu: " + str(image2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = rpu2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = rpu2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4rpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4rpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2rpu [key2task] = rpu2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2rpu-set-by-get2imageinfo4rpu to: " + str(image2rpu [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4rpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4rpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; rpu2image4xi-using-by-get2image4rpu: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2rpu = create2image4rpu()
            message2print = "{(Step 1F/33) initialized" + "; image2rpu-initialized-by-get2imageinfo4rpu to: " + str(image2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2rpu = rpu2image4xi
                message2print = "{(Step 1G/33) got" + "; image2rpu-got-by-get2imageinfo4rpu: " + str(image2rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4rpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2rpu [key2task] = rpu2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2rpu-set-by-get2imageinfo4rpu to: " + str(image2rpu [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4rpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2rpu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2rpu-yielded-by-get2image4rpu: " + str(image2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2rpu-returning-by-get2imageinfo4rpu: " + str(image2rpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4rpu', 
    #  'METHOD': 'create2image4rpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4rpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4rpu', 
    #  'METHOD': 'create3image4rpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2rpu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4rpu" usage = "
#
#   print('(Step 3A/33) OF RPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR RPU', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4rpu()                                                                       
#       print('(Step 3B/33) OF RPU2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR RPU: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF RPU2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR RPU', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4rpu(task4supportedimageformats2rpu=None):  

    '''
    gets supported image formats for rpu
    '''
    
    global rpu2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4rpu"
    method2print = "get2supportedimageformats4rpu"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4rpu-started: " + str(get2supportedimageformats4rpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4rpu at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2rpu-passed-to-get2supportedimageformats4rpu: " + str(task4supportedimageformats2rpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2rpu is None:
        task4supportedimageformats2rpu = {} 
        task4supportedimageformats2rpu['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2rpu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2rpu['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2rpu = {} 
    try: 
        if rpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2rpu = create2image4rpu()                                                                          # create rpu2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2rpu-initialized-by-get2supportedimageformats4rpu to: " + str(image2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR RPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2rpu = rpu2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2rpu-got-by-get2supportedimageformats4rpu: " + str(supportedimageformats2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2rpu = create2image4rpu()
            message2print = "{(Step 1F1/33) initialized" + "; image2rpu-initialized-by-get2supportedimageformats4rpu to: " + str(image2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR RPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2rpu = rpu2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2rpu-got-by-get2supportedimageformats4rpu: " + str(supportedimageformats2rpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2rpu-yielded-by-get2image4rpu: " + str(supportedimageformats2rpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2rpu-returning-by-get2supportedimageformats4rpu: " + str(supportedimageformats2rpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2rpu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4rpu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF RPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR RPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2rpu = wait2image4rpu()
#       keys2response = response4wait2rpu.keys()
#       print('(Step 16B/700) OF RPU2IMAGE4XI5TEST - WAITED 2D IMAGE FOR RPU ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2rpu[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4rpu at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4rpu', 'METHOD': 'create2image4rpu'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4rpu at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR RPU: ', wait2rpu)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4rpu(task4wait2image=None):

    '''
    waits rpu2image4rpu
    '''
    
    global rpu2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4rpu"
    method2print = "wait2image4rpu"

    message2print = "{(Step 1A1/33) started" + "; wait2image4rpu-started: " + str(wait2image4rpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4rpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if rpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            rpu2image4xi = create2image4rpu()
            
            message2print = "{(Step 1B/33) created" + "; rpu2image4xi-created-by-wait2image4rpu: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4rpu = None
            try:
                queue2image4rpu = rpu2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4rpu-got-by-wait2image4rpu to: " + str(queue2image4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; rpu2xi-yielded-by-wait2image4rpu: " + str(rpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4rpu = 0
            try:                    
                size2image4rpu = queue2image4rpu.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4rpu-got-by-wait2image4rpu to: " + str(size2image4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; rpu2xi-yielded-by-wait2image4rpu: " + str(rpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4rpu = True       
            try:                    
                empty2image4rpu= queue2image4rpu.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4rpu-got-by-wait2image4rpu to: " + str(empty2image4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4rpu-yielded-by-wait2image4rpu: " + str(empty2image4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4rpu = False       
            try:                                
                full2image4rpu = queue2image4rpu.full()
                message2print = "{(Step 1F/33) got" + "; full2image4rpu-got-by-wait2image4rpu to: " + str(full2image4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4rpu-yielded-by-wait2image4rpu: " + str(full2image4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; rpu2image4xi-finished-by-wait2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR RPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; rpu2image4xi-used-by-wait2image4rpu: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4rpu = None
            try:
                queue2image4rpu = rpu2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4rpu-got-by-wait2image4rpu to: " + str(queue2image4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; rpu2xi-yielded-by-wait2image4rpu: " + str(rpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4rpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4rpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4rpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4rpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4rpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4rpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4rpu-warned-by-wait2image4rpu: " + " ** " + str(queue2image4rpu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4rpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4rpu = 0        
            try:                    
                size2queue4rpu = queue2image4rpu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4rpu-got-by-wait2image4rpu to: " + str(size2queue4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                rpu2image4xi ['SIZE2QUEUE'] = size2queue4rpu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4rpu to: " + str(rpu2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4rpu-yielded-by-wait2image4rpu: " + str(size2queue4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4rpu = False
            try:                    
                empty2queue4rpu = queue2image4rpu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4rpu-got-by-wait2image4rpu to: " + str(empty2queue4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                rpu2image4xi ['EMPTY2QUEUE'] = empty2queue4rpu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4rpu to: " + str(rpu2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4rpu-yielded-by-wait2image4rpu: " + str(empty2queue4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4rpu = False
            try:                                
                full2queue4rpu = queue2image4rpu.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4rpu-got-by-wait2image4rpu to: " + str(full2queue4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                rpu2image4xi ['FULL2QUEUE'] = full2queue4rpu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4rpu to: " + str(rpu2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4rpu-yielded-by-wait2image4rpu: " + str(full2queue4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4rpu = None
            try:                                
                state2queue4rpu = queue2image4rpu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4rpu-got-by-wait2image4rpu to: " + str(state2queue4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                rpu2image4xi ['STATE2QUEUE'] = state2queue4rpu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4rpu to: " + str(rpu2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4rpu-yielded-by-wait2image4rpu: " + str(state2queue4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; rpu2image4xi-finished-by-wait2image4rpu: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            rpu2image4rpu = create2image4rpu()
            message2print = "{(Step 1R/33) got" + "; rpu2image4xi-got-by-wait2image4rpu to: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4rpu = None
            try:
                queue2image4rpu = rpu2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4rpu-got-by-wait2image4rpu to: " + str(queue2image4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4rpu-yielded-by-wait2image4rpu: " + str(queue2image4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4rpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4rpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4rpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4rpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4rpu-warned-by-wait2image4rpu: " + " ** " + str(queue2image4rpu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4rpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4rpu = 0                             
            try:                    
                size2image4rpu = queue2image4rpu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4rpu-got-by-wait2image4rpu to: " + str(size2image4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4rpu-yielded-by-wait2image4rpu: " + str(size2image4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4rpu = True          
            try:                    
                empty2image4rpu = queue2image4rpu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4rpu-got-by-wait2image4rpu to: " + str(empty2image4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; rpu2xi-yielded-by-wait2image4rpu: " + str(rpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4rpu = False          
            try:                                
                full2image4rpu= queue2image4rpu.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4rpu-got-by-wait2image4rpu to: " + str(full2image4rpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4rpu-yielded-by-wait2image4rpu: " + str(full2image4rpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; rpu2image4xi-finished-by-wait2image4rpu: " + str(rpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            rpu2image4rpu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; rpu2image4xi-yielded-by-wait2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; rpu2image4xi-returning-by-wait2image4rpu: " + str(rpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return rpu2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "RPU2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for RPU 2D images" usage = "2D Image4Xi of RPU"
#>

# =================================================================================================
class RPU2Image4Xi(object): 
    ''' 
    RPU2Image4Xi offers methods for RPU 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2rpu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-RPU2Image4Xi" usage = "
    #   print('(Step 5A/33) OF RPU2IMAGE4XI5TEST - CREATING IMAGE4RPU OBJECT ', time4xi())     
    #   try:
    #       imageobject2rpu = RPU2Image4Xi()
    #       print('(Step 5B/33) OF RPU2IMAGE4XI5TEST - CREATED IMAGE4RPU OBJECT: ', imageobject2rpu)                                  
    #   except:
    #       print('(Step 5Z/33) OF RPU2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4RPU OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2rpu = None):
        '''
        _init__ method initializes RPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "RPU2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-RPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; rpu2image2xi-of-RPU2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-rpu2image2xi-of-RPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2rpu-sharing-by-rpu2image2xi-of-RPU2Image4Xi: " + str(RPU2Image4Xi.classvariable2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2rpu-passed-to-init-of-RPU2Image4Xi: " + str(task4imagecreate2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2rpu-passing-to-create2image4rpu-by-init-of-RPU2Image4Xi: " + str(task4imagecreate2rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        rpu2image2xi = create2image4rpu(task4imagecreate2rpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4rpu-performed-by-init-of-RPU2Image4Xi: " + str(create2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4rpu at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; rpu2image2xi-of-RPU2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { RPU2Image4Xi : __init__ ; (Step 3A/33) activated; rpu2image2xi-of-RPU2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-rpu2image2xi-of-RPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2rpu-shared-by-rpu2image2xi-of-RPU2Image4Xi: " + str(RPU2Image4Xi.classvariable2rpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-RPU2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-RPU2Image4Xi" usage = "
    #   print('(Step 23A/33) OF RPU2IMAGE4XI5TEST - TRYING TO CREATE RPU IMAGE', time4xi()) 
    #   try:
    #       xi2rpu = RPU2Image4Xi()                            
    #       image2rpu = xi2rpu.create2image()                                  
    #       print('(Step 23B/33) OF RPU2IMAGE4XI5TEST - CREATED RPU IMAGE: ', image2rpu)
    #   except:
    #       print('(Step 23Z/33) OF RPU2IMAGE4XI5TEST *** FAILED TO CREATE RPU IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of RPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-RPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4rpu-by-create2image-of-RPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        rpu2image2xi = create2image4rpu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4rpu-performed-by-create2image-of-RPU2Image4Xi: " + str(create2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4rpu at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; rpu2image2xi-returning-by-create2image-of-RPU2Image4Xi: " + str(rpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4rpu',
        #  'METHOD': 'create2image4rpu'},
        #  'WIDTH': 800 ...

        return rpu2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-RPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF RPU2IMAGE4XI - TRYING TO SET RPU IMAGE')
    #   try:
    #       cc2xi = RPU2Image4Xi()                            
    #       image2rpu = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF RPU4XI5TEST - SET RPU IMAGE: ', image2rpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF RPU4XI5TEST *** FAILED TO SET RPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image RPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-RPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4rpu-by-set2image-of-RPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        rpu2image2xi = set2image4rpu(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4rpu-performed-by-set2image-of-RPU2Image4Xi: " + str(set2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4rpu at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; rpu2image2xi-returning-by-set2image-of-RPU2Image4Xi: " + str(rpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4rpu', 'METHOD': 'create2image4rpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4rpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4rpu',
        #  'METHOD': 'create2buffer4rpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4rpu at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return rpu2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-RPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF RPU2IMAGE4XI - TRYING TO GET RPU 2D IMAGE')
    #   try:
    #       cc2xi = RPU2Image4Xi()                            
    #       image2rpu = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF RPU4XI5TEST - GOT RPU IMAGE: ', image2rpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF RPU4XI5TEST *** FAILED TO GET RPU 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of RPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        rpu2image2xi = get2imageinfo4rpu(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; rpu2image2xi-returning-by-get2imageinfo-of-RPU2Image4Xi: " + str(rpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4rpu', 'METHOD': 'create2image4rpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4rpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4rpu',
        #  'METHOD': 'create2buffer4rpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4rpu at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return rpu2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-RPU2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF RPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR RPU')
    #  try:
    #      cc2xi = RPU2Image4Xi()                            
    #      supportedimageformats2rpu = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF RPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR RPU: ', supportedimageformats2rpu)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF RPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR RPU***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of RPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2rpu = get2supportedimageformats4rpu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2rpu-returning-by-get2imageinfo-of-RPU2Image4Xi: " + str(supportedimageformats2rpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2rpu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-RPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2rpu = {}
    #   try:
    #       xi2rpu = RPU2Image4Xi()    
    #       response4read2rpu = xi2rpu.read2image()
    #       print('(Step 127B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2RPU: ', response4read2rpu) 
    #   except:
    #       print('(Step 127Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4rpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads RPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-RPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4rpu-by-read2image-of-RPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2rpu = read2image4rpu(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4rpu-performed-by-read2image-of-RPU2Image4Xi: " + str(read2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4rpu at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2rpu-returning-by-read2image-of-RPU2Image4Xi: " + str(response4readimage2rpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2rpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-RPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2rpu = RPU2Image4Xi() 
    #   ...
    #   response4map2rpu = {}
    #   try:
    #       response4map2rpu = xi2rpu.map2image()
    #       print('(Step 127B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2RPU: ', response4map2rpu) 
    #   except:
    #       print('(Step 127Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4rpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps RPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-RPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4rpu-by-map2image-of-RPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2rpu = map2image4rpu(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4rpu-performed-by-map2image-of-RPU2Image4Xi: " + str(map2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4rpu at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2rpu-returning-by-map2image-of-RPU2Image4Xi: " + str(response4mapimage2rpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2rpu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-RPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF RPU DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2rpu.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF RPU2IMAGE4XI5TEST - RELEASED 2D IMAGE OF RPU DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF RPU DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF RPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF RPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2rpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF RPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF RPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF RPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF RPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases RPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-RPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4rpu-by-release2image-of-RPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        rpu2image2xi = release2image4rpu(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4rpu-performed-by-release2image-of-RPU2Image4Xi: " + str(release2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; rpu2image2xi-returning-by-release2image-of-RPU2Image4Xi: " + str(rpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return rpu2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-RPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF RPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF RPU DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2rpu.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF RPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF RPU DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF RPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF RPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2rpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF RPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF RPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF RPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF RPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains RPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-RPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4rpu-by-retain-of-RPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        rpu2image2xi = retain2image4rpu(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4rpu-performed-by-retain2image-of-RPU2Image4Xi: " + str(retain2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4rpu at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; rpu2image2xi-returning-by-retain2image-of-RPU2Image4Xi: " + str(rpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4rpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4rpu', 
        #  'METHOD': 'create2image4rpu'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return rpu2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-RPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF RPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR RPU ', time4xi()) 
    #   print('=' *33)      
    #   wait2rpu = {}
    #   try:
    #       response4wait2Dimage = xi2rpu.wait2image()
    #       print('(Step 122B/700) OF RPU2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR RPU ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR RPU ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits RPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-RPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-RPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4rpu-by-wait2image-of-RPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        rpu2image2xi = wait2image4rpu(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4rpu-performed-by-wait2image-of-RPU2Image4Xi: " + str(wait2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4rpu at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; rpu2image2xi-returning-by-wait2image-of-RPU2Image4Xi: " + str(rpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return rpu2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-RPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2rpu = {}
    #   try:
    #       response4write2rpu = xi2rpu.write2image()
    #       print('(Step 127B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2RPU: ', response4write2rpu) 
    #   except:
    #       print('(Step 127Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4rpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes RPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-RPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4rpu-by-write2image-of-RPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2rpu = write2image4rpu(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4rpu-performed-by-write2image-of-RPU2Image4Xi: " + str(write2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4rpu at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2rpu-returning-by-write2image-of-RPU2Image4Xi: " + str(response4writeimage2rpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2rpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-RPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2rpu = {}
    #   try:
    #       response4copy2rpu = xi2rpu.copy2image()
    #       print('(Step 127B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2RPU: ', response4copy2rpu) 
    #   except:
    #       print('(Step 127Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4rpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies RPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-RPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4rpu-by-copy2image-of-RPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2rpu = copy2image4rpu(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4rpu-performed-by-copy2image-of-RPU2Image4Xi: " + str(copy2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4rpu at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2rpu-returning-by-copy2image-of-RPU2Image4Xi: " + str(response4copyimage2rpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2rpu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-RPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2rpu.copy2imagetobuffer()
    #       print('(Step 127B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2RPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4rpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-RPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4rpu-by-copy2imagetobuffer-of-RPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2rpu = copy2imagetobuffer4rpu(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4rpu-performed-by-copy2imagetobuffer-of-RPU2Image4Xi: " + str(copy2imagetobuffer4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4rpu at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2rpu-returning-by-copy2imagetobuffer-of-RPU2Image4Xi: " + str(response4copyimage2rpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2rpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-RPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF RPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2rpu.copy2buffertoimage()
    #       print('(Step 127B/700) OF RPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2RPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF RPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4rpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-RPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4rpu-by-copy2buffertoimage-of-RPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2rpu = copy2buffertoimage4rpu(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4rpu-performed-by-copy2buffertoimage-of-RPU2Image4Xi: " + str(copy2buffertoimage4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4rpu at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2rpu-returning-by-copy2buffertoimage-of-RPU2Image4Xi: " + str(response4copyimage2rpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2rpu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-RPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF RPU2IMAGE4XI5TEST - TRYING TO TRACE RPU: ', time4xi())      
    #   print('=' *33)      
    #   trace2rpu = None
    #   try:
    #       trace2rpu = trace2image4rpu() 
    #       print('(Step 15B/25) OF RPU2IMAGE4XI5TEST - TRACE2RPU: ', trace2rpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4rpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4rpu', 'METHOD': 'trace2image4rpu'}
    #    except:
    #       print('(Step 15Z1/25) OF RPU2IMAGE4XI5TEST *** FAILED TO TRACE RPU: ', trace2rpu)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces RPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-RPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4rpu-by-trace2image-of-RPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4rpu(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4rpu-performed-by-trace2image-of-RPU2Image4Xi: " + str(trace2image4rpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4rpu at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-RPU2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4rpu-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4rpu',
        #  'METHOD': 'trace2image4rpu'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-RPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF RPU2IMAGE4XI - TRYING TO SET RPU IMAGE')
    #   try:
    #       cc2xi = RPU2Image4Xi()                            
    #       image2rpu = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF RPU4XI5TEST - SET RPU IMAGE: ', image2rpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF RPU4XI5TEST *** FAILED TO SET RPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of RPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "RPU2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-RPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-RPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2rpu4image = {} 
        try:
            callback2rpu4image = set2imagecallback4rpu (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2rpu4image" + "; callback2rpu4image-set-by-set2imagecallback-of-RPU2Image4Xi to: " + str(callback2rpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4rpu at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2rpu4image-failed-by-set2imagecallback-of-RPU2Image4Xi: " + str(callback2rpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2rpu4image-returning-by-set2imagecallback-of-RPU2Image4Xi: " + str(callback2rpu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4rpu at 0x0000000003AB31C8> 
                                             
        return callback2rpu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-RPU2Image4Xi" usage = "
    #   try:
    #       print(xi2rpu.sense())
    #       print('(Step 24B/33) OF RPU2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF RPU2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses RPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'RPU2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; RPU2Image4Xi-sensing-by-sense-of-RPU2Image4Xi: " + str(RPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'rpu2image4xi.RPU2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-RPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <rpu2image4xi.RPU2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2rpu-of-RPU2Image4Xi'] = RPU2Image4Xi.classvariable2rpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-RPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; RPU2Image4Xi-sensed-by-sense-of-RPU2Image4Xi: " + str(RPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'rpu2image4xi.RPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-RPU2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <rpu2image4xi.RPU2Image4Xi object at 0x0000000004188400>, 'classvariable2rpu-of-RPU2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-RPU2Image4Xi" usage = "
    #   print('(Step 25A/33) OF RPU2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2rpu.reason())
    #       print('(Step 25B/33) OF RPU2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF RPU2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons RPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'RPU2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; RPU2Image4Xi-reasoning-by-reason-of-RPU2Image4Xi: " + str(RPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'rpu2image4xi.RPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-RPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <rpu2image4xi.RPU2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-RPU2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-RPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; RPU2Image4Xi-reasoned-by-reason-of-RPU2Image4Xi: " + str(RPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'rpu2image4xi.RPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-RPU2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <rpu2image4xi.RPU2Image4Xi object at 0x0000000004188400>, 'time-reason-of-RPU2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-RPU2Image4Xi" usage = "
    #   print('(Step 26A/33) OF RPU2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2rpu.ace())
    #       print('(Step 26B/33) OF RPU2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF RPU2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces RPU2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'RPU2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-RPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; RPU2Image4Xi-acing-by-ace-of-RPU2Image4Xi: " + str(RPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'rpu2image4xi.RPU2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-RPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <rpu2image4xi.RPU2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-RPU2Image4Xi'] = RPU2Image4Xi.classvariable2rpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-RPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; RPU2Image4Xi-aced-by-ace-of-RPU2Image4Xi: " + str(RPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'rpu2image4xi.RPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-RPU2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-RPU2Image4Xi': 4, 'object8xi': <rpu2image4xi.RPU2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   rpu2image4xi5test.txt
#   sense-rpu2image4xi5test.xml
#   reason-rpu2image4xi5test.xml
#   ace-rpu2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of rpu2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
